;
; RAW DISK DUMP UTILITY FOR C1P/UK101
;
; DUMPS THE CONTENTS OF A DISK AND SENDS OUTPUT
; TO THE CASSETTE PORT AS ASCII HEX. IT WILL
; CAPTURE ALL DATA ON THE DISK INCLUDING THE
; TRACK/SECTOR FORMATTING INFORMATION.
;
; DUMPS EVERY BYTE ENCOUNTERED ON A TRACK
; STARTING AT THE INDEX PULSE.
;
; NOTE: DOS FORMATTED TRACKS WILL CONTAIN
; SPURIOUS CHARACTERS BEFORE AND AFTER SECTORS
; AND CAN BE IGNORED.
;
;
; HISTORY:
;
; 0.4  CODE REVAMPED. DISABLED ERROR CHECK
;      (USELESS FOR DOS DISKS). REMOVED
;      MESSAGES FROM DUMP OUTPUT.
;
; 0.3  INCREASED DELAYS FOR USE ON FASTER
;      MACHINES.
;
; 0.21 FIXED ERROR COUNT. CHANGED BUFFER
;      BUFFER SIZE TO 9 PAGES AND FILL CHAR
;      TO $00.
;
; 0.2  ACIA IS RESET BEFORE EACH TRACK READ.
;      PARITY ERRORS CHECKED. READING STOPS
;      AT THE NEXT INDEX PULSE, SO NO MORE
;      EXTRANEOUS DATA IN TRACK DUMPS.
;
; 0.1  1ST RELEASE
;


ORG	=	$0400

;MAXTRK	=	40	; # TRACKS TO READ
;MAXPAG	=	9	; # PAGES TO READ
FILLCH	=	$00	; FILL CHARACTER

BUFFER	=	$2000	; TRACK BUFFER

; OSI MONITOR ROM ROUTINES
LOADF	=	$203	; LOAD FLAG
SAVEF	=	$205	; SAVE FLAG

INKEY	=	$FD00	; GET CHAR FROM KEYBOARD
;OUTPUT	=	$FFEE	; PUT CHAR TO SCREEN/ACIA

DISK	=	$C000	; DISK CONTROLLER

CR		=	$0D
LF		=	$0A

;Mark's OSI Screen Driver Program...
;storage
XPOS	=	$01A0
YPOS	=	XPOS+1
TMPCHR	=	XPOS+2
UNDER	=	XPOS+3
ESCSET	=	XPOS+4
DIRECT	=	XPOS+5
OPTION	=	XPOS+6
ROWLEN	=	XPOS+7	;length of video
BASEHI	=	XPOS+8	;video start addr hi
BASELO	=	XPOS+9	;video start addr lo
YSIZE	=	XPOS+10	;height of display
XSIZE	=	XPOS+11	;width of display
MACHINE =   XPOS+12 ;C1/C2 flag 00=C2/4/8
MAXTRK  =   XPOS+13	;40 for 5.25, 77 for 8
MAXPAG  =   XPOS+14	;9 for 5.25, 14 for 8
ACIACTL =   XPOS+15	;controls ACIA divisor, format
DRVNUM  =   XPOS+16 ;drive# 0-3


CURSOR	=	$EE ;USES EE,EF
LOAD	=	$EC ;USES EC,ED
MLPLER	=	$E9
PROD	=	$E8


	*=	ORG
	LDA #$00	;determine machine type
	BIT $DF00
	BMI	*+3
	BIT $FFA9
	STA MACHINE
	JSR SCRINIT
	LDA #$15	;8N1
	STA ACIACTL
	JSR INZACIA
	LDA #$00
	STA DRVNUM
	LDA #40
	STA	MAXTRK
	LDA #9
	STA MAXPAG



TOP
	LDA	#0
	STA	LOADF	; LOAD OFF
	STA	SAVEF	; SAVE OFF
	JSR DRWMENU

MENUL0
	JSR INKEY
	JSR OUTCHR
	CMP #$31
	BNE *+5
	JMP RAWREAD
	CMP #$32
	BNE *+5
	JMP DOSREAD
	CMP #$33
	BNE *+5
	JMP	SETDRV
	CMP #$34
	BNE *+5
	JMP DRVTYPE
	CMP #$35
	BNE *+5
	JMP ACIADIV
	CMP #$36
	BNE MENUL0
	JMP	($FFFC)	;Exit

DRVTYPE
	JSR PRINT
	.BYTE $D,$A,'Enter your disk drive type (8) inch or (5).25 inch? >',0
	JSR INKEY
	JSR OUTCHR
	CMP #$38
	BEQ SETDRV8
	CMP #$35
	BEQ SETDRV5
INPERR
	LDA #'?
	JSR OUTCHR
	JMP TOP

SETDRV5
	LDA #40
	STA	MAXTRK
	LDA #9
	STA MAXPAG
	JMP TOP

SETDRV8
	LDA #77
	STA	MAXTRK
	LDA #14
	STA MAXPAG
	JMP TOP

ACIADIV
	JSR PRINT
	.BYTE $D,$A,'Set (N)ormal or 16x (F)aster? >',0
	JSR INKEY
	JSR OUTCHR
	CMP #'F
	BEQ DIV1
	CMP #'f
	BEQ DIV1
	CMP #'N
	BEQ DIV16
	CMP #'n
	BEQ DIV16
	JMP INPERR

DIV1
	LDA #$14  ;8N1 div 1

DIV16=*+1
	BIT $15A9	 ;8N1 div 16 (normnal)
	STA ACIACTL
	JSR INZACIA
	JMP TOP

SETDRV
	JSR PRINT
	.BYTE $D,$A,'Select drive (A), (B), (C), (D) ? >',0
	JSR INKEY
	JSR OUTCHR
	CMP #$60
	BCC *+4
	EOR #$20
	CMP #'A
	BCC SETERR
	CMP #'E
	BCS SETERR
	SBC #'@		;carry is clear so val -1
	STA DRVNUM
	JMP TOP
SETERR
	JMP INPERR

DOSREAD
	JMP INPERR

RAWREAD
;-----------------------------------------------------------
	JSR	SAVEOFF

	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'RAW DISK DUMP 0.4',CR,LF,CR,LF
	.BYTE	'DUMP RAW TRACK DATA FROM',CR,LF
	.BYTE	'DISKS TO CASSETTE',CR,LF,CR,LF,0

	JSR	PRINT
	.BYTE	'WRITE PROTECT ALL DISKS!',CR,LF
	.BYTE	CR,LF
	.BYTE	'INSERT DISK TO BE DUMPED',0

	JSR	ANYKEY

	JSR	INITPIA
	JSR	RESERR
	JSR SELDRV

	LDA	#0
	STA	TRK
LOOP	
	LDA	TRK
	CMP	MAXTRK
	BEQ	QUIT

	PHA
	JSR	READMSG
	PLA
	BNE	LOOP2

	JSR	TZERO	; STEP TO TRACK 0
	BCS	NOTRK
	JMP	LOOP3

LOOP2	
	JSR	TNEXT	; STEP TO NEXT TRACK

LOOP3	
	JSR	RDTRK
	BCS NODRIVE

	JSR	SAVMSG
	LDA	MAXPAG
	JSR	SAVBUF
	INC	TRK
	JMP	LOOP

NODRIVE
	JSR PRINT
	.BYTE CR,LF, 'ERROR Waiting for disk index!',0
	JSR ANYKEY
	JMP TOP

NOTRK
	JSR PRINT
	.BYTE CR,LF,CR,LF, 'ERROR Seeking Track 0',0
	JSR ANYKEY
	JMP TOP	

QUIT
;	JSR	SHOWERR
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	'DONE ... REMOVE DISK',CR,LF,0
	JSR	INKEY
	JMP TOP

; SAVE BUFFER CONTENTS (A)

SAVBUF	
	PHA
	JSR	RESPTR
	JSR	SAVEON
	PLA
	STA	$FF
	LDA	#0
	STA	CNT
	STA	CNT+1
	TAY
	TAX
	BEQ	SAVBUF5

SAVBUF1	
	CPX	#0
	BNE	SAVBUF2
	JSR	CRLF
	LDX	#16
	BNE	SAVBUF3

SAVBUF2	
	LDA	#' '
	JSR	OUTPUT
SAVBUF3	
	LDA	($FD),Y
	JSR	PHEXA
	INC	CNT
	BNE	SAVBUF4
	INC	CNT+1
SAVBUF4	
	DEX
	INY
	BNE	SAVBUF1
	INC	$FE
	DEC	$FF
SAVBUF5	
	LDA	$FF
	BNE	SAVBUF1
	JSR	CRLF
	JMP	SAVEOFF

; INIT POINTERS, CLEAR BUFFER

INIT	
	JSR	RESPTR	; SET POINTERS
	LDY	#0
	LDA	#FILLCH
INIT1	
	STA	($FD),Y
	INY
	BNE	INIT1
	INC	$FE
	DEC	$FF
	BNE	INIT1
	JMP	RESPTR

; RESET BUFFER POINTERS

RESPTR	
	LDA	MAXPAG
	STA	$FF
	LDA	#<BUFFER
	STA	$FD
	LDA	#>BUFFER
	STA	$FE
	RTS

; INIT DISK CONTROLLER PIA

INITPIA	
	LDY	#0
	LDA #$40
	STY	DISK+1	; SELECT DDRA
	STA	DISK	; SET PORTA TO ALL INPUTS except PA6
	LDX	#4		; 0000 0100
	STX	DISK+1	; SELECT PORTA
	STA DISK    ; SET PB6 HIGH
	STY	DISK+3	; SELECT DDRB
	DEY
	STY	DISK+2	; SET PORTB TO ALL OUTPUTS
	STX	DISK+3	; SELECT PORTB
	STY	DISK+2	; SET PORTB OUTPUTS HIGH
	RTS


;SELECT DRIVE PB5 PA6  DRIVE (1-4)
;              0   0	#4
;              0   1	#3
;              1   0	#2
;              1   1	#1

SELDRV
	LDA DRVNUM
	LSR A
	TAY
	BCC *+5
	LDA #$00
	BIT $40A9
	STA DISK
	LDA DISK+2
	ORA #$20
	CPY #$01
	BCC	*+4
	AND #$DF
	STA DISK+2
	RTS


; STEP TO TRACK 0

TZERO
	LDA MAXTRK	;max num tracks
	ADC #$08	;plus a few more
	STA TMP		;MAX times to step before abort
	LDA DISK+2
	AND #$FB	; 1111 1011
	BNE	TZERO3	; DIR=INWARDS

TZERO1
	DEC TMP
	BNE TZERO5
	SEC
	RTS
TZERO5	
	LDA	#2		; 0000 0010
	BIT	DISK	; TEST 'TRK0' PIN
	BNE	TZERO2	; AT TRACK 0
	CLC
	RTS

TZERO2	
	LDA	DISK+2	; DIR=OUTWARDS (TO TRK0)
	ORA #$04
TZERO3	
	STA	DISK+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DISK+2	; SET 'STEP' PIN LOW
	JSR	DELAY2
	ORA	#8		; 0000 1000
	STA	DISK+2	; SET 'STEP' PIN HIGH
	LDX	#$30	; 60 MS
	JSR	DELAY
	BEQ	TZERO1	; ALWAYS

; STEP TO NEXT TRACK

TNEXT	
	LDA DISK+2
	AND	#$FB	; 1111 1011
				; DIR=INWARDS
	STA	DISK+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DISK+2	; SET 'STEP' LOW
	JSR	DELAY2
	ORA	#8		; 0000 1000
	STA	DISK+2	; SET 'STEP' HIGH
	LDX	#$30	; 60MS
	JMP	DELAY




; READ CURRENT TRACK TO BUFFER

RDTRK	
	JSR	INIT
	LDA DISK+2  ; PRESERVE DISK SEL BIT
	AND	#$7F	; 0111 1111
	STA	DISK+2	; LOAD DISK HEAD
	LDX	#$FF	; 320MS
	JSR	DELAY
RDTRK1	
	BIT	DISK	; WAIT START OF
	BMI	RDTRK1	; INDEX PULSE
RDTRK2	
	BIT	DISK	; WAIT END OF
	BPL	RDTRK2	; INDEX PULSE
	LDA	#3		; RESET ACIA
	STA	DISK+$10
	LDA	#$58	; DIVIDE BY 1, 8E1, RTS HIGH
	STA	DISK+$10
	LDY	#0		; READ UNTIL ALL PAGES DONE
RDTRK3	
	JSR	RDBYT
	BCS	RDTRK4	; OR END-OF-TRACK
	STA	($FD),Y
	INY
	BNE	RDTRK3
	INC	$FE
	DEC	$FF
	BNE	RDTRK3
RDTRK4
	LDA DISK+2
	ORA #$80	
	STA	DISK+2	; UNLOAD DISK HEAD
	CLC
	RTS

; DELAY 1.25MS PER LOOP

DELAY	
	LDY	#$F8
DELAY1	
	DEY
	BNE	DELAY1
	EOR	$FF,X
	DEX
	BNE	DELAY
	RTS

DELAY2	
	JSR	DELAY21
DELAY21	
	RTS

; INPUT CHAR FROM DISK, CY SET IF END-OF-TRACK

RDBYT	
	BIT	DISK	; TEST FOR INDEX PULSE
	BPL	RDBYT2
	LDA	DISK+$10
	LSR	A
	BCC	RDBYT
	BIT	DISK+$10 ; TEST PARITY ERROR
	BVS	RDBYT3
	LDA	DISK+$11 ; READ BYTE
	CLC
	RTS

RDBYT2	
	LDA	#FILLCH
	SEC
	RTS

RDBYT3	
	INC	ERRCNT
	BNE	RDBYT4
	INC	ERRCNT+1
RDBYT4	
	LDA	DISK+$11 ; READ BYTE
	CLC
	RTS

; PRINT IN-LINE STRING

PRINT	
	PLA
	STA	$FD
	PLA
	STA	$FE
	LDY	#1
PRINT1	
	LDA	($FD),Y
	BEQ	PRINT2
	JSR	OUTPUT
	INY
	BNE	PRINT1
	INC $FE
	BNE PRINT1
PRINT2	
	TYA
	SEC
	ADC	$FD
	STA	$FD
	BCC	PRINT3
	INC	$FE
PRINT3	
	JMP	($FD)

; PRINT HEX WORD (A,X)

PHEX	
	JSR	PHEXA
	TXA

; PRINT HEX BYTE (A)

PHEXA	
	PHA
	LSR	A
	LSR	A
	LSR	A
	LSR	A
	JSR	PHEXA1
	PLA
PHEXA1	
	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	PHEXA2
	CLC
	ADC	#7
PHEXA2	
	JMP	OUTPUT

; PRINT DECIMAL (A)

PDECA	
	TAX
	LDA	#0

; PRINT DECIMAL (A,X)

PDEC	
	STX	PDEC8
	STA	PDEC8+1
	LDA	#0
	LDX	#2
PDEC1	
	STA	PDEC10,X
	DEX
	BPL	PDEC1
	LDY	#16
	PHP
	SEI
	SED
PDEC2	
	ASL	PDEC8
	ROL	PDEC8+1
	LDX	#2
PDEC3	
	LDA	PDEC10,X
	ADC	PDEC10,X
	STA	PDEC10,X
	DEX
	BPL	PDEC3
	DEY
	BNE	PDEC2
	PLP
	LDX	#6
	LDA	#0
	STA	PDEC9
PDEC4	
	LDY	#3
	LDA	#0
PDEC5	
	ASL	PDEC10+2
	ROL	PDEC10+1
	ROL	PDEC10
	ROL	A
	DEY
	BPL	PDEC5
	TAY
	BNE	PDEC6
	CPX	#1
	BEQ	PDEC6
	LDY	PDEC9
	BEQ	PDEC7
PDEC6	INC	PDEC9
	ORA	#$30
	JSR	OUTPUT
PDEC7	DEX	
	BNE	PDEC4
	RTS

PDEC8	.WORD	0
PDEC9	.BYTE	0
PDEC10	.BYTE	0,0,0

CRLF	
	LDA	#CR
	JSR	OUTPUT
	LDA	#LF
	JMP	OUTPUT

ANYKEY	
	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'PRESS ANY KEY WHEN READY',CR,LF,0
	JMP	INKEY

READMSG	
	PHA
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	'READING TRACK ',0
	PLA
	JMP	PDECA

SAVMSG	
	JSR	PRINT
	.BYTE	' ...SAVING',CR,LF,0
	RTS

SHOWERR	
	JSR	PRINT
	.BYTE	CR,LF,CR,LF,'ERRORS ',0
	LDA	ERRCNT+1
	LDX	ERRCNT
	JSR	PDEC
	JMP	CRLF


RETRY
	JSR PRINT
	.BYTE $D,$A,'Error reading track. (A)bort, (R)etry, (I)gnore? >',0
	JSR INKEY
	JSR OUTCHR
	CMP #'A
	BEQ RETRY0
	CMP #'R
	BEQ RETRY0 
	CMP #'I
	BNE RETRY
RETRY0
	RTS	 

SAVEON	
	LDA	#1
	STA	SAVEF
	RTS

SAVEOFF	
	LDA	#0
	STA	LOADF
	STA	SAVEF
	RTS

RESERR	
	LDA	#0
	STA	ERRCNT
	STA	ERRCNT+1
	RTS





DRWMENU
	JSR PRINT
	.BYTE $1B,$48,$1B,$45                                        
	.BYTE $D,$A,$DD,$84,$84,$84,$84,$84,$84,$84,$84
	.BYTE $84,$84,$84,$84,$84,$84,$84,$84,$84
	.BYTE $84,$84,$84,$84,$DE,$0D,$0A  
	.BYTE $8B,'OSI Serial Disk Dump ',$8C,$D,$A          
	.BYTE $8B,'-------------------- ',$8C,$D,$A          
	.BYTE $8B,'1. Dump Raw Image    ',$8C,$D,$A
	.BYTE $8B,'2. Dump OSI DOS Image',$8C,$D,$A
	.BYTE $8B,'3. Select Drive      ',$8C,$D,$A      
	.BYTE $8B,'4. Set Drive Type    ',$8C,$D,$A      
	.BYTE $8B,'5. Set Serial Divisor',$8C,$D,$A      
	.BYTE $8B,'6. Exit              ',$8C,$D,$A          
	.BYTE $8B,'                     ',$8C,$D,$A
	.BYTE $8B,'Drv=',$0
	LDA DRVNUM
	CLC
	ADC #$41
	JSR	OUTPUT
	LDA	MAXPAG
	CMP #$0A
	BCS DRWMN1
	JSR PRINT
	.BYTE '/5.25',0
	JMP DRWMN2
DRWMN1
	JSR PRINT
	.BYTE '/8   ',0
DRWMN2
	JSR PRINT
	.BYTE ' Baud @ 1',0
   	LDA ACIACTL
	LSR A
	BCC DRWMN3
	JSR PRINT
	.BYTE 'x ',0
	JMP DRWMN4
DRWMN3
	JSR PRINT
	.BYTE '6x'
DRWMN4	
	JSR PRINT
	 .BYTE $8C,$D,$A	                 
	.BYTE $DC,$83,$83,$83,$83,$83,$83,$83,$83  
	.BYTE $83,$83,$83,$83,$83,$83,$83,$83,$83,$83
	.BYTE $83,$83,$83,$DF,$D,$A                
	.BYTE $84,$84,$84,$C2,$0                
	RTS


INZACIA
	BIT MACHINE
	BMI INZAC1
    LDA #$03	;reset ACIA
	STA $FC00
	LDA ACIACTL	;ACIA /16 8n1 TX,RX irq=OFF
	STA $FC00
	RTS
INZAC1
	LDA #$03
	STA $F000
	LDA ACIACTL
	STA $F000
	RTS


FRACIA   	 ;read from ACIA carry set on abort return value in A
	BIT MACHINE
	BMI FRAC1

	LDA #$02 
	STA $DF00
	LDA #$10 
	BIT $DF00
	BNE ACIARET 
	LDA $FC00
	LSR A    
	BCC FRACIA 
	LDA $FC01
	CLC      
ACIARET
	RTS
FRAC1
	LDA #$FD 
	STA $DF00
	LDA #$EF 
	BIT $DF00
	BNE ACIARET 
	LDA $F000
	LSR A    
	BCC FRACIA 
	LDA $F001
	CLC       
	RTS

TOACIA    
	PHA
	BIT MACHINE
	BMI TOAC1
	       
	LDA $FC00   ;wait for TxEmpty
	LSR A     
	LSR A     
	BCC *-5   
	PLA       
	STA $FC01 
	RTS       
TOAC1
	LDA $F000 
	LSR A     
	LSR A     
	BCC *-5   
	PLA       
	STA $F001 
	RTS 


OUTPUT
	PHA
	LDA SAVEF
	BEQ OUTPUT2
	PLA
	PHA
	JSR TOACIA
OUTPUT2
	PLA
	JSR OUTCHR
	RTS




;---------------------------------------------------------------------	


	;Mark's OSI Screen Driver Program...
	;based on Heath/Zenith H29
	;
	; ESC A cursor up
	; ESC B cursor down
	; ESC C cursor forward
	; ESC D cursor backwards
	; ESC H move cursor to home
	; ESC I scroll UP
	; ESC n report cursor pos (not implemented)
	; ESC j save cursor pos
	; ESC k restore previously saved cursor pos
	; ESC Y## move cursor to column#,row#
	; ESC - backtab
	; ESC E Clear Screen
	; ESC b Erase from start of screen to cursor
	; ESC J Erase from cursor to end of display
	; ESC I Erase line
	; ESC o Erase from cursor to beginning of line 
	; ESC K Erase for cursor to end of line
	; ESC L Insert Line
	; ESC M Delete Line
	; ESC N Delete character
	; ESC @ Enter insert character mode
	; ESC O End Insert character mode
	; 


OUTCHR=*
	JMP SCREEN
SCRINIT=*
	LDX #$05

	LDA C2INIT-1,X
	BIT MACHINE
	BPL *+5
	LDA C1INIT-1,X
	STA ROWLEN-1,X
	DEX
	BNE SCRINIT+2

	LDX #$00
	STX XPOS
	STX YPOS
	STX OPTION
	STX ESCSET
	BIT MACHINE
	BMI *+7
	LDX #$01
	STX $DE00
	JSR CLS
	RTS
SCREEN=*
	STA TMPCHR
	PHA
	TYA
	PHA
	TXA
	PHA
	LDA TMPCHR
	BIT ESCSET
	BPL *+5
	JMP ESCRTN
	CMP #$7F
	BNE *+5
	JMP DEL
	CMP #$20
	BCS ISCHAR
	LDX #$06
	DEX
	BPL *+5
	JMP ENDCHR
	CMP ACTIVE,X
	BNE *-9
	TXA
	ASL A
	TAX
	LDA LOOKUP,X
	PHA
	LDA LOOKUP+1,X
	PHA
	RTS
	LDA TMPCHR
ISCHAR=*
	LDX #$05
	DEX
	BMI *+13
	CMP SPCL,X
	BNE *-6
	LDA FIX,X
	STA TMPCHR
	BIT OPTION
	BPL NRMCHR
	PHA
	LDY XSIZE
	DEY
	CPY XPOS
	BNE OKSERT
	LDA OPTION
	AND #$7F
	STA OPTION
	PLA
	JMP NRMCHR
OKSERT=*
	LDX CURSOR+1
	STX LOAD+1
	LDX CURSOR
	INX
	STX LOAD
	DEY
	LDA (CURSOR),Y
	STA (LOAD),Y
	CPY XPOS
	BNE *-8
	INC XPOS
	PLA
	STA (CURSOR),Y
	JMP ENDCHR
NRMCHR=*
	LDY XPOS
	STA (CURSOR),Y
	INY
	CPY XSIZE
	BCC *+15
	BIT OPTION
	BVS STRUDR
	LDY #$00
	STY XPOS
	JMP FSCROL
	INC XPOS
STRUDR=*
	LDY XPOS
	LDA (CURSOR),Y
	STA UNDER
	LDA #$5F
	STA (CURSOR),Y
ENDCHR=*
	PLA
	TAX
	PLA
	TAY
	PLA
	RTS
CRET=*
	JSR RESTOR
	LDA #$00
	STA XPOS
	JMP STRUDR
LFEED=*
	JSR RESTOR
FSCROL=*
	LDY YSIZE
	DEY
	CPY YPOS
	BEQ *+11
	INC YPOS
	JSR SETCUR
	JMP STRUDR
	LDA YPOS
	PHA
	LDA #$00
	STA DIRECT
	STA YPOS
	JSR SCROLL
	PLA
	STA YPOS
	JSR SETCUR
	JMP STRUDR
RSCROL=*
	JSR RESTOR
	LDY YPOS
	BEQ *+11
	DEC YPOS
	JSR SETCUR
	JMP STRUDR
	LDA #$FF
	STA DIRECT
	JSR SCROLL
	JMP STRUDR
SCROLL=*
	LDA YSIZE
	CLC
	SBC YPOS
	TAX
	BIT DIRECT
	BPL *+8
	LDY YSIZE
	DEY
	BNE *+5
	LDY YPOS
	STY TMPCHR
	JSR SETCUR+3
LOOPSC=*
	LDY TMPCHR
	BIT DIRECT
	BPL *+4
	DEY
	BIT $C8
	STY TMPCHR
	JSR GENPOS
	STA LOAD+1
	STY LOAD
	LDY XSIZE
	DEY
	LDA (LOAD),Y
	STA (CURSOR),Y
	DEY
	BPL *-5
	LDA LOAD+1
	STA CURSOR+1
	LDA LOAD
	STA CURSOR
	DEX
	BNE LOOPSC
	LDA #$20
	LDY XSIZE
	DEY
	STA (LOAD),Y
	BPL *-3
	RTS
GENPOS=*
	STY MLPLER
	LDA #$00
	LDY #$08
GENLOP=*
	LSR MLPLER
	BCC GEN1
	CLC
	ADC ROWLEN
GEN1=*
	ROR A
	ROR PROD
	DEY
	BNE GENLOP
	PHA
	LDA PROD
	CLC
	ADC BASELO
	TAY
	PLA
	ADC BASEHI
	RTS
SETCUR=*
	LDY YPOS
	JSR GENPOS
	STA CURSOR+1
	STY CURSOR
	RTS
CLREOL=*
	LDY XPOS
	LDA #$20
	STA (CURSOR),Y
	INY
	CPY XSIZE
	BNE *-6
	RTS
CLRBOL=*
	LDY #$FF
	LDA #$20
	INY
	STA (CURSOR),Y
	CPY XPOS
	BNE *-6
	RTS
BAKSPC=*
	JSR RESTOR
	LDY XPOS
	DEY
	BMI *+5
	DEC XPOS
	JMP STRUDR
TAB=*
	JSR RESTOR
	LDA XPOS
	AND #$78
	CLC
	ADC #$08
	CMP XSIZE
	BCS *+5
	STA XPOS
	JMP STRUDR
DEL=*
	LDA XSIZE
	SEC
	SBC XPOS
	TAX
	LDY XPOS
DLOOP=*
	DEX
	BEQ ENDDEL
	INY
	LDA (CURSOR),Y
	DEY
	STA (CURSOR),Y
	INY
	BNE DLOOP
ENDDEL=*
	LDA #$20
	LDY XSIZE
	DEY
	STA (CURSOR),Y
	JMP STRUDR
CLREOS=*
	LDA YPOS
	PHA
	LDY XPOS
	JSR CLREOL+3
	INC YPOS
	JSR SETCUR
	LDY #$00
	LDA YPOS
	CMP YSIZE
	BCC *-17
	PLA
	STA YPOS
	JSR SETCUR
	JMP STRUDR
UPCRSR=*
	JSR RESTOR
	LDA YPOS
	BEQ *+8
	DEC YPOS
	JSR SETCUR
	JMP STRUDR
DNCRSR=*
	JSR RESTOR
	LDY YPOS
	INY
	CPY YSIZE
	BCS *+8
	INC YPOS
	JSR SETCUR
	JMP STRUDR
RTCRSR=*
	JSR RESTOR
	LDY XSIZE
	DEY
	CPY XPOS
	BEQ *+5
	INC XPOS
	JMP STRUDR
HOME=*
	JSR RESTOR
	LDA #$00
	STA XPOS
	STA YPOS
	JSR SETCUR
	JMP STRUDR
SRTIRT=*
	LDA #$80
	ORA OPTION
	BNE ENDIRT+5
ENDIRT=*
	LDA #$7F
	AND OPTION
	STA OPTION
	JMP ENDCHR
ESCAPE=*
	LDA #$80
	STA ESCSET
	JMP ENDCHR
CLS=*
	LDX YSIZE
CLSLOP=*
	DEX
	BMI CLSEND
	TXA
	TAY
	JSR SETCUR+3
	LDA #$20
	LDY XSIZE
	DEY
	STA (CURSOR),Y
	BNE *-3
	BEQ CLSLOP
CLSEND=*
	JMP SETCUR
CLR=*
	LDY #$00
	STY YPOS
	STY XPOS
	JSR CLS
	JMP STRUDR
RESTOR=*
	LDY XPOS
	LDA UNDER
	STA (CURSOR),Y
	RTS
ESCRTN=*
	LDY #$00
	BIT ESCSET
	BVC *+8
	STY ESCSET
	JMP (LOAD)
	STY ESCSET
	CMP #$40
	BCC ESCCHK
	CMP #$50
	BCS ESCCHK
	SEC
	SBC #$40
	ASL A
	TAX
	LDA ESCTBL,X
	PHA
	LDA ESCTBL+1,X
	PHA
	RTS
ESCCHK=*
	LDX #$05
	CMP ESC2TB,X
	BEQ *+8
	DEX
	BPL *-6
	JMP ISCHAR
	TXA
	ASL A
	TAX
	LDA ESCTBL+$20,X
	PHA
	LDA ESCTBL+$21,X
	PHA
	RTS
DELLIN=*
	LDX #$00
ADDLIN=*+1
	BIT $FFA2
	STX DIRECT
	JSR RESTOR
	LDY #$00
	STY XPOS
	LDY YSIZE
	DEY
	CPY YPOS
	BNE *+8
	JSR CLREOL
	JMP STRUDR
	JSR SCROLL
	JSR SETCUR
	JMP STRUDR
MVCRSR=*
	JSR RESTOR
	JSR POP
	LDA TMPCHR
	SEC
	SBC #$20
	CMP YSIZE
	BCS *+5
	STA YPOS
	JSR SETCUR
	JSR POP
	LDA TMPCHR
	SEC
	SBC #$20
	TAY
	CPY YSIZE
	BCC *+6
	LDY YSIZE
	DEY
	STY XPOS
	JMP STRUDR
POP=*
	PLA
	CLC
	ADC #$01
	STA LOAD
	PLA
	ADC #$00
	STA LOAD+1
	LDA #$C0
	STA ESCSET
	JMP ENDCHR
BAKTAB=*
	JSR RESTOR
	LDA XPOS
	BEQ *+15
	AND #$78
	CMP XPOS
	BCC *+5
	SEC
	SBC #$08
	STA XPOS
	JMP STRUDR
ZOTEND=*
	JSR CLREOL
	JMP STRUDR
ZOTBEG=*
	JSR CLRBOL
	JMP STRUDR
ZOTLIN=*
	JSR CLRBOL
	JSR CLREOL
	JMP STRUDR
CLRBOS=*
	JSR CLRBOL
	LDX YPOS
	BEQ *+5
	JSR CLSLOP
	JMP STRUDR
LOOKUP=*
	.DBYTE CLR-1,BAKSPC-1,ESCAPE-1,TAB-1,CRET-1,LFEED-1
ACTIVE=*
	.BYTE $0C,$08,$1B,$09,$0D,$0A,$FF
ESCTBL=*
	.DBYTE SRTIRT-1,UPCRSR-1,DNCRSR-1,RTCRSR-1
	.DBYTE BAKSPC-1,CLR-1,ISCHAR-4,ISCHAR-4,HOME-1
	.DBYTE RSCROL-1,CLREOS-1,ZOTEND-1,ADDLIN-1
	.DBYTE DELLIN-1,DEL-1,ENDIRT-1,MVCRSR-1
	.DBYTE BAKTAB-1,ZOTBEG-1,CLRBOS-1,ZOTLIN-1
	.DBYTE ESCAPE-1
ESC2TB=*
	.BYTE 'Y-ob1',$1B
SPCL=*
	.BYTE $7C,$7D,$7E,$7F,$60
FIX=*
	.BYTE $7D,$7C,$7F,$7E,$DC

; ROWLEN,BASEHI,BASELO,YSIZE,XSIZE
C2INIT=*
	.BYTE $40,$D0,$80,$1D,$40
C1INIT=*
	.BYTE $20,$D0,$85,$18,$18

	BRK

CNT	
	.WORD	0
ERRCNT	
	.WORD	0
TRK	
	.BYTE	0
TMP
	.BYTE   0


	.END

