/***************************************************************************

		  6502.cpp

		  A 6502 emulator core
        (only standard 6502 code, ERR code length added)

***************************************************************************/

#include <string.h>
#include <stdio.h>
#include "6502.h"
#include "w_osi.h"
#include "w_io.h"
#include "disk_io.h"
#include "keyboard.h"
#include "Monitor.h"
#include "fileio.h"
#include "6850.h"


Acia g_SerialPort; //one serial port class for cassette I/O
DiskCtrl g_DiskPort; 

/* Global Variables */

unsigned char BCDtoDEC[256];
unsigned char DECtoBCD[256];
unsigned int TEMP;     /* Temporary register used for calculating EA */
unsigned int TEMP1;    /* Temporary register */
unsigned int TEMP2;    /* Temporary register */

int  work;     		  /* Work variable for <0 comparisons */

void (*readMEMORY[256])();      /* Array of pointers to read functions  */
void (*writeMEMORY[256])();     /* Array of pointers to write functions */

unsigned int memoryaddress;
unsigned char readvalue;
unsigned char writevalue;
unsigned char *CODEptr;            // Last Code ptr adress
int IRQ =0, NMI =0;
unsigned int eff_adr=0;
BOOL is_peek_access = FALSE;




void (*Inst[256])();            /* Array of pointers to functions */
void (*Mode[256])();            /* Array of modes for each instruction */
int Ticks[256];                 /* Array of tick counts for each instr. */
char Op[256][6];
CPUMODEL R;       					 /* Define Registers Type */

//*******************************************************************
// Main 6502 Execute Call
//
// paramaters:
//             none
//
//             gClocks       - sum of all executed cpu cycles
//             CODEptr       - pointer to next instruction
//
//*******************************************************************

int _6502(void)
{
	int clocks =0;
	
	if (NMI)	
	{
		nmi_cpu(); 
		NMI=0;
		clocks += 7;
	}
	
	if (IRQ) 
	{
		if(R.P.I == 0)
		{
			irq_cpu();
			IRQ=0;
			clocks += 7;
		}
	}

	// Fetch opcode (Fast version direct access)

	CODEptr=RAMptr+R.PC.W; R.OP=*CODEptr;
	++R.PC.W&=0xFFFF;
	CODEptr++;                      // Point to next Code byte

	// Execute opcode
	Inst[R.OP]();
	clocks+=Ticks[R.OP];
	
	gClocks += clocks;

	return clocks;
}


unsigned char Peek(unsigned int address)
/***************************************************************************
		  Peek
		  Returns the value at address passed.
		  Controls also the appropriate hardware emulation.
***************************************************************************/
{
	is_peek_access = TRUE;
	
	memoryaddress=address&0xFFFF;
	readMEMORY[memoryaddress>>8]();
	
	is_peek_access = FALSE;

	return(readvalue);
}

unsigned int Deek(unsigned int address)
/***************************************************************************
		  FAST Deek
		  Returns the value at address , without I/O control
		  Word access for 6502 codes only direct from RAM
***************************************************************************/
{
	return(*((unsigned int *)(RAMptr+address)));
}


void Poke(unsigned int address, unsigned char value)
/***************************************************************************
		  Poke
		  Copies 'value' into memory location 'adress'
		  Controls also the appropriate hardware emulation.
***************************************************************************/
{
	memoryaddress=address&0xFFFF;
	writevalue=value;
	PokeFlag=TRUE;
	writeMEMORY[memoryaddress>>8]();
}

void NULLREAD()
/***************************************************************************
		  NULLREAD
		  Returns High byte address
***************************************************************************/
{
  readvalue= (BYTE) (memoryaddress>>8);
}

void RAMREAD()
/***************************************************************************
		  RAMREAD
		  Returns byte value
***************************************************************************/
{
  readvalue=RAMptr[memoryaddress];
}
void ROMBASICREAD()
/***************************************************************************
		  ROMBASICREAD        $A000-BFFF
		  Returns byte value
***************************************************************************/
{
  readvalue=RAMptr[memoryaddress];
}
void ROMKERNALREAD()
/***************************************************************************
		  ROMKERMALREAD       $F800-FFFF
		  Returns byte value
***************************************************************************/
{
  readvalue=RAMptr[memoryaddress];
}

/***************************************************************************
		  ACIA1READ       Seial Port I/O
		  Returns byte value
***************************************************************************/
void ACIA1READ()
{
	g_SerialPort.Read(memoryaddress, &readvalue);
}

/***************************************************************************
		  ACIA1WRITE       Seial Port I/O
		  Returns byte value
***************************************************************************/
void ACIA1WRITE()
{
	g_SerialPort.Write(memoryaddress, writevalue);
}


/***************************************************************************
		  PIAREAD            
		  
***************************************************************************/
void PIAREAD()
{
}

/***************************************************************************
		  PIAWRITE              

***************************************************************************/
void PIAWRITE()
{
}

/***************************************************************************
		  DISKREAD              

***************************************************************************/
void DISKREAD()
{
	g_DiskPort.Read(memoryaddress, &readvalue);
}
/***************************************************************************
		  DISKWRITE              

***************************************************************************/
void DISKWRITE()
{
	g_DiskPort.Write(memoryaddress, writevalue);

}

void VIDCREAD()
/***************************************************************************
		  VIDCREAD          $D000-D7FF, $E000-E7FF
		  Returns byte value
***************************************************************************/
{
  readvalue=RAMptr[memoryaddress];

  //upper nibble NULL READ
  if (gMachineClass == C2Machine && memoryaddress >= 0xE000)
	readvalue |= 0xE0; 
}



void NULLWRITE()
/***************************************************************************
		  NULLWRITE
		  Does nothing
***************************************************************************/
{}

void RAMWRITE()
/***************************************************************************
		  RAMWRITE
		  Writes byte value
***************************************************************************/
{
  RAMptr[memoryaddress]=writevalue;
}

/***************************************************************************
	VIDCTRLWD
	Writes byte value to the video control register on 540 board
		bit 0 0=32, 1=64
		bit 1 0=tone off, 1=tone on (542 keyboard)
		bit 2 0=color off, 1=color on 
		bit 3 1=enable 15Khz AC Home control output
***************************************************************************/
void VIDCTRLWD()
{

	int old = 0;
	
	if ( (writevalue & 0x02) != gSoundCtrl ? 4 : 0)
	{
		gSoundCtrl = (writevalue & 0x02 ) ? 1 : 0;
		//TODO handle enable sound
	}

	if (gScreenMode & WIDTH_MASK )
		old |= 1;

	if (gScreenMode & COLOR_MASK)
		old |= 4;

	if ( (writevalue & 0x05) != old)
	{
	
		if (writevalue & 0x04)
			gScreenMode |= COLOR_MASK;
		else
			gScreenMode &= ~COLOR_MASK;

		if (writevalue & 0x01)
			gScreenMode |= WIDTH_MASK;
		else
			gScreenMode &= ~WIDTH_MASK;

		//notify display change
		PostMessage(ghWnd_Main, WM_SCREEN_UPD, 0, 0);
	}
}

/***************************************************************************
		  VIDCTRLRD
		  Reads byte value from video control register on 540 board
		  bit 7 = video timing clock (60Hz?) 0 = low, 1 = high
***************************************************************************/
void VIDCTRLRD()
{
	readvalue = (( gClocks % 16700) > 8349) ? '\xDF' : '\x5F';
}


/***************************************************************************
		  VIDCWRITE
		  Writes byte value to the text video memory
***************************************************************************/
void VIDCWRITE()
{
  PokeFlag=TRUE;
  RAMptr[memoryaddress]=writevalue;
  OSI_VIDC(memoryaddress); // Show Char
}

void GDCWRITE()
/***************************************************************************
		  GDCCWRITE
		  Writes byte value to the graphik memory
***************************************************************************/
{
  PokeFlag=TRUE;
  RAMptr[memoryaddress]=writevalue;
  if (GEnable) 
	OSI_GDC(memoryaddress); // Show Pixel
}

void addr_abs()
/***************************************************************************

		  addr_abs      3Bytes  ()

		  Set R.EA to the address contained in the next two unsigned chars

***************************************************************************/
{
  R.EA=(unsigned int)(*CODEptr|(*(CODEptr+1)<<8));
  eff_adr=R.EA;
  R.PC.W++; R.PC.W++;
}



void addr_absx()
/***************************************************************************

		  addr_absx      3Bytes  (X)

		  Set R.EA to the address contained in the next two unsigned chars plus X

***************************************************************************/
{
  R.EA=(unsigned int)(*CODEptr|(*(CODEptr+1)<<8))+ R.X;
  eff_adr=R.EA;
  R.PC.W++; R.PC.W++;
}





void addr_absy()
/***************************************************************************

		  addr_absy       3Bytes (Y)

		  Set R.EA to the address contained in the next two unsigned chars plus Y

***************************************************************************/
{
  R.EA=(unsigned int)(*CODEptr|(*(CODEptr+1)<<8))+ R.Y;
  eff_adr=R.EA;
  R.PC.W++; R.PC.W++;
}



void addr_immediate()
/***************************************************************************

		  addr_immediate  2Bytes ()

		  Set R.EA to address of next unsigned char

***************************************************************************/
{
  R.EA=R.PC.W;
  eff_adr=R.EA;
  R.PC.W++;
}





void addr_implied()
/***************************************************************************

		  addr_implied     1Byte ()

		  No data for this command

***************************************************************************/
{
  eff_adr=0;
}





void addr_indirect()
/***************************************************************************

		  addr_indirect     3Bytes ()

***************************************************************************/
{
  unsigned int LT;

  LT=(unsigned int)(*CODEptr|(*(CODEptr+1)<<8));
  R.EA=(unsigned int)Peek(LT)|((unsigned int)Peek(LT+1)<<8);
  eff_adr=R.EA;
  R.PC.W++; R.PC.W++;
}




void addr_indx()
/***************************************************************************

		  addr_indx        2Bytes (X)

		  The second unsigned char of the instruction is added to the X register,
		  discarding the carry.  The result points to a location on page 0
		  which contains the 8 low-order bits of the R.EA.  The next unsigned char
		  contains the 8 high-order bits.

***************************************************************************/
{
  unsigned int LT;

  LT=(unsigned int)(*CODEptr)+R.X;
  LT &= 0x00FF;       /* Drop any carry */
  R.EA=(unsigned int)(*(RAMptr+LT)|(*(RAMptr+LT+1)<<8));
  eff_adr=R.EA;
  R.PC.W++;
}





void addr_indy()
/***************************************************************************

		  addr_indy      2Bytes (Y)

		  The second unsigned char of the instruction points to a location in page 0.
		  The contents of this memory location is added to the Y register,
		  the result being the 8 low-order bits of the R.EA.  The carry is
		  added to the contents of the next page 0 location, the results
		  being the 8 high-order bits of the R.EA.

***************************************************************************/
{
  unsigned int LT;

  LT=(unsigned int)(*CODEptr);
  R.EA=(unsigned int)(*(RAMptr+LT)|(*(RAMptr+LT+1)<<8))+R.Y;
  eff_adr=R.EA;
  R.PC.W++;
}




void addr_relative()
/***************************************************************************

		  addr_relative    2Bytes ()

		  Add next unsigned char as SIGNED to current address.

***************************************************************************/
{
  R.EA=(unsigned int)(*CODEptr);
  if(R.EA & 0x80) R.EA -= 0x100;
  eff_adr=R.EA;
  R.PC.W++;
}




void addr_zpage()
/***************************************************************************

		  addr_zpage      2Bytes (Z)

		  Set R.EA to next unsigned char (page 0)

***************************************************************************/
{
  R.EA=(unsigned int)(*CODEptr);
  eff_adr=R.EA;
  R.PC.W++;
}



void addr_zpagex()
/***************************************************************************

		  addr_zpagex     2Bytes (Z,X)

		  Set R.EA to next unsigned char (page 0) plus X (carry dropped)

***************************************************************************/
{
  R.EA=(unsigned int)(*CODEptr)+R.X;
  R.EA &= 0x00FF;   /* Drop any carry */
  eff_adr=R.EA;
  R.PC.W++;
}




void addr_zpagey()
/***************************************************************************

		  addr_zpagey    2Bytes (Z,Y)

		  Set R.EA to next unsigned char (page 0) plus Y (carry dropped)

***************************************************************************/
{
  R.EA=(unsigned int)(*CODEptr)+R.Y;
  R.EA &= 0x00FF;   /* Drop any carry */
  eff_adr=R.EA;
  R.PC.W++;
}




unsigned int Disassemble(char *S, unsigned int A)
/***************************************************************************

		  Disassemble

		  Disassembles data at address A and stores the disassembly in
		  string S.  Returns the number of unsigned chars disassembled.
***************************************************************************/
{
  unsigned char o;
  int inc=1;
  unsigned char tt;

  o=Peek(A);
  lstrcpy(S,Op[o]);
  if(Mode[o]==addr_abs) { inc=3; A++; wsprintf(S,"%s $%04X",Op[o],Deek(A)&0xFFFF); }
  else if(Mode[o]==addr_absx) { inc=3; A++; wsprintf(S,"%s $%04X,X",Op[o],Deek(A)&0xFFFF); }
  else if(Mode[o]==addr_absy) { inc=3; A++; wsprintf(S,"%s $%04X,Y",Op[o],Deek(A)&0xFFFF); }
  else if(Mode[o]==addr_immediate) { inc=2; A++; wsprintf(S,"%s #$%02X",Op[o],Peek(A)&0xFF); }
  else if(Mode[o]==addr_implied) { inc=1; wsprintf(S,"%s",Op[o]); }
  else if(Mode[o]==addr_indirect) { inc=3; A++; wsprintf(S,"%s ($%04X)",Op[o],Deek(A)&0xFFFF); }
  else if(Mode[o]==addr_indx) { inc=2; A++; wsprintf(S,"%s ($%02X,X)",Op[o],Peek(A)&0xFF); }
  else if(Mode[o]==addr_indy) { inc=2; A++; wsprintf(S,"%s ($%02X),Y",Op[o],Peek(A)&0xFF); }
  else if(Mode[o]==addr_relative){ inc=2; A++; tt=(BYTE)(Peek(A)&0xFF); A++; wsprintf(S,"%s $%04X",Op[o],A+((tt<0x80)? tt:(tt-256)));}
  else if(Mode[o]==addr_zpage) { inc=2; A++; wsprintf(S,"%s $%02X",Op[o],Peek(A)&0xFF); }
  else if(Mode[o]==addr_zpagex) { inc=2; A++; wsprintf(S,"%s $%02X,X",Op[o],Peek(A)&0xFF); }
  else if(Mode[o]==addr_zpagey) { inc=2; A++; wsprintf(S,"%s $%02X,Y",Op[o],Peek(A)&0xFF); }

  return(inc);
}


void i_adc()
/***************************************************************************

		  adc

***************************************************************************/
{
  unsigned char LB;
  unsigned int LW;

  Mode[R.OP]();
  LB=Peek(R.EA);
  

  if (R.P.D)
  {
	LW = (R.A & 0x0f) + (LB & 0x0f) +(R.P.C);
	if (LW >= 10)
		LW = (LW - 10) | 0x10;
	LW += (R.A & 0xf0) + (LB & 0xf0);

	R.P.V = ((R.A ^ LB) & 0x80) && (( (unsigned int)R.A ^ LW) & 0x80) ? 1 : 0;

	if (LW > 0x9f)
		LW += 0x60;
	R.P.C = LW > 0xff ? 1 : 0;
	R.A = (unsigned char) LW;


  }
  else
  {
	  LW=R.A+LB+(R.P.C);                     /* Carry ? */
	  if(LW>0xFF)
	  {
		 R.P.C=1;
		 LW -= 0x100;
	  }
	  else R.P.C=0;
	  if(LW>0x7F) 
		R.P.V=1;
	  else 
		R.P.V=0;     								/* Overflow ? */
	  R.A=(unsigned char)LW;
  }
  if(R.A) R.P.Z=0;
  else R.P.Z=1;              						/* Zero result ? */
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;      								/* Negative result ? */
}





void i_and()
/***************************************************************************

		  and

***************************************************************************/
{
  unsigned char LB;

  Mode[R.OP]();
  LB=Peek(R.EA);
  R.A &= LB;
  if(R.A==0) R.P.Z=1;
  else R.P.Z=0;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_asl()
/***************************************************************************

        asl

***************************************************************************/
{
  unsigned int LW;

  Mode[R.OP]();
  LW=Peek(R.EA);
  if(LW & 0x80) R.P.C=1;
  else R.P.C=0;
  LW=LW << 1;
  LW &= 0xFF;
  Poke(R.EA,(unsigned char)LW);
  if(LW) R.P.Z=0;
  else R.P.Z=1;
  if(LW & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_asla()
/***************************************************************************

		  asla

***************************************************************************/
{
  if(R.A & 0x80) R.P.C=1;
  else R.P.C=0;
  R.A=(unsigned char)(R.A << 1);
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_bcc()
/***************************************************************************

		  bcc

***************************************************************************/
{
  if(R.P.C==0)
  {
	 Mode[R.OP]();
	 R.PC.W += R.EA;
  }
  else
  {
	 R.PC.W++;
  }
}





void i_bcs()
/***************************************************************************

		  bcs

***************************************************************************/
{
  if(R.P.C==1)
  {
    Mode[R.OP]();
    R.PC.W += R.EA;
  }
  else
  {
    R.PC.W++;
  }
}





void i_beq()
/***************************************************************************

		  beq

***************************************************************************/
{
  if(R.P.Z==1)
  {
    Mode[R.OP]();
    R.PC.W += R.EA;
  }
  else
  {
	 R.PC.W++;
  }
}





void i_bit()
/***************************************************************************

		  bit

***************************************************************************/
{
  unsigned char LB;

  Mode[R.OP]();
  LB=Peek(R.EA);
  if(LB & R.A) R.P.Z=0;
  else R.P.Z=1;
  if(LB & 0x80) R.P.N=1;
  else R.P.N=0;
  if(LB & 0x40) R.P.V=1;
  else R.P.V=0;
}





void i_bmi()
/***************************************************************************

		  bmi

***************************************************************************/
{
  if(R.P.N==1)
  {
	 Mode[R.OP]();
	 R.PC.W += R.EA;
  }
  else
  {
	 R.PC.W++;
  }
}





void i_bne()
/***************************************************************************

		  bne

***************************************************************************/
{
  if(R.P.Z==0)
  {
	 Mode[R.OP]();
	 R.PC.W += R.EA;
  }
  else
  {
	 R.PC.W++;
  }
}





void i_bpl()
/***************************************************************************

        bpl

***************************************************************************/
{
  if(R.P.N==0)
  {
	 Mode[R.OP]();
    R.PC.W += R.EA;
  }
  else
  {
	 R.PC.W++;
  }
}





void i_brk()
/***************************************************************************

		  brk

***************************************************************************/
{
  unsigned char LB;

  R.PC.W++;
  Poke(0x0100+R.S,(unsigned char)(R.PC.B.h)); R.S--;
  Poke(0x0100+R.S,(unsigned char)(R.PC.B.l)); R.S--;
  LB=(unsigned char)(R.P.C
	 + (((unsigned char)R.P.Z)<<1)
    + (((unsigned char)R.P.I)<<2)
    + (((unsigned char)R.P.D)<<3)
	 + (((unsigned char)R.P.B)<<4)
	 + 0x20
    + (((unsigned char)R.P.V)<<6)
	 + (((unsigned char)R.P.N)<<7));
  Poke(0x0100+R.S,(unsigned char)LB); R.S--;
  R.P.Z=0; R.P.D=0; R.P.V=0; R.P.N=0;
  R.PC.B.l=Peek(0xFFFE);
  R.PC.B.h=Peek(0xFFFF);
}





void i_bvc()
/***************************************************************************

        bvc

***************************************************************************/
{
  if(R.P.V==0)
  {
    Mode[R.OP]();
    R.PC.W += R.EA;
  }
  else
  {
    R.PC.W++;
  }
}





void i_bvs()
/***************************************************************************

		  bvs

***************************************************************************/
{
  if(R.P.V==1)
  {
	 Mode[R.OP]();
	 R.PC.W += R.EA;
  }
  else
  {
	 R.PC.W++;
  }
}





void i_clc()
/***************************************************************************

        clc

***************************************************************************/
{
  R.P.C=0;
}





void i_cld()
/***************************************************************************

		  cld

***************************************************************************/
{
  R.P.D=0;
}





void i_cli()
/***************************************************************************

		  cli

***************************************************************************/
{
  R.P.I=0;
}





void i_clv()
/***************************************************************************

        clv

***************************************************************************/
{
  R.P.V=0;
}





void i_cmp()
/***************************************************************************

		  cmp

***************************************************************************/
{
  unsigned int TW;

  Mode[R.OP]();
  R.P.C=0; R.P.Z=0; R.P.N=0;
  TW=0x100+R.A-Peek(R.EA);
  if(TW>0xff) R.P.C=1;
  if((TW&0xff)==0) R.P.Z=1;
  if((TW&0xff)>0x7F) R.P.N=1;
}




void i_cpx()
/***************************************************************************

		  cpx

***************************************************************************/
{
  unsigned int TW;

  Mode[R.OP]();
  R.P.C=0; R.P.Z=0; R.P.N=0;
  TW=0x100+R.X-Peek(R.EA);
  if(TW>0xff) R.P.C=1;
  if((TW&0xff)==0) R.P.Z=1;
  if((TW&0xff)>0x7F) R.P.N=1;
}





void i_cpy()
/***************************************************************************

		  cpy

***************************************************************************/
{
  unsigned int TW;

  Mode[R.OP]();
  R.P.C=0; R.P.Z=0; R.P.N=0;
  TW=0x100+R.Y-Peek(R.EA);
  if(TW>0xff) R.P.C=1;
  if((TW&0xff)==0) R.P.Z=1;
  if((TW&0xff)>0x7F) R.P.N=1;
}





void i_dec()
/***************************************************************************

		  dec

***************************************************************************/
{
	Mode[R.OP]();
	
	work=Peek(R.EA)-1;
	if(work<0) 
	work+=0x100;
	Poke(R.EA,(unsigned char)work);
	work &= 0xff;

	if(work) 
		R.P.Z=0;
	else 
		R.P.Z=1;
	if(work & 0x80) 
		R.P.N=1;
	else 
		R.P.N=0;
}





void i_dex()
/***************************************************************************

		  dex

***************************************************************************/
{
  R.X--;
  if(R.X) R.P.Z=0;
  else R.P.Z=1;
  if(R.X & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_dey()
/***************************************************************************

		  dey

***************************************************************************/
{
  R.Y--;
  if(R.Y) R.P.Z=0;
  else R.P.Z=1;
  if(R.Y & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_eor()
/***************************************************************************

		  eor

***************************************************************************/
{
  Mode[R.OP]();
  R.A ^= Peek(R.EA);
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_inc()
/***************************************************************************

        inc

***************************************************************************/
{
	Mode[R.OP]();

	TEMP=Peek(R.EA)+1;
	if(TEMP>0xFF) 
		TEMP-=0x100;
	Poke(R.EA,(unsigned char)TEMP);
	TEMP &= 0xff;

	if(TEMP) R.P.Z=0;
	else R.P.Z=1;
	if(TEMP & 0x80) R.P.N=1;
	else R.P.N=0;


}





void i_inx()
/***************************************************************************

        inx

***************************************************************************/
{
  R.X++;
  if(R.X) R.P.Z=0;
  else R.P.Z=1;
  if(R.X & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_iny()
/***************************************************************************

		  iny

***************************************************************************/
{
  R.Y++;
  if(R.Y) R.P.Z=0;
  else R.P.Z=1;
  if(R.Y & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_jmp()
/***************************************************************************

		  jmp

***************************************************************************/
{
  Mode[R.OP]();
  R.PC.W=R.EA;
}





void i_jsr()
/***************************************************************************

		  jsr

***************************************************************************/
{
  R.PC.W++;
  Poke(0x0100+R.S,R.PC.B.h); R.S--;
  Poke(0x0100+R.S,R.PC.B.l); R.S--;
  R.PC.W--;
  Mode[R.OP]();
  R.PC.W=R.EA;
}





void i_lda()
/***************************************************************************

        lda

***************************************************************************/
{
  Mode[R.OP]();
  R.A=Peek(R.EA);
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_ldx()
/***************************************************************************

        ldx

***************************************************************************/
{
  Mode[R.OP]();
  R.X=Peek(R.EA);
  if(R.X) R.P.Z=0;
  else R.P.Z=1;
  if(R.X & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_ldy()
/***************************************************************************

        ldy

***************************************************************************/
{
  Mode[R.OP]();
  R.Y=Peek(R.EA);
  if(R.Y) R.P.Z=0;
  else R.P.Z=1;
  if(R.Y & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_lsr()
/***************************************************************************

		  lsr

***************************************************************************/
{
  Mode[R.OP]();
  TEMP=Peek(R.EA);
  if(TEMP & 0x01) R.P.C=1;
  else R.P.C=0;
  TEMP=(TEMP >> 1);
  Poke(R.EA,(unsigned char)TEMP);
  TEMP &= 0xFF;
  if(TEMP) R.P.Z=0;
  else R.P.Z=1;
  if(TEMP & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_lsra()
/***************************************************************************

		  lsra

***************************************************************************/
{
  TEMP=R.A;
  if(TEMP & 0x01) R.P.C=1;
  else R.P.C=0;
  TEMP=(TEMP >> 1);
  R.A=(unsigned char)TEMP;
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_nop()
/***************************************************************************

		  nop

***************************************************************************/
{
}





void i_ora()
/***************************************************************************

		  ora

***************************************************************************/
{
  Mode[R.OP]();
  R.A |= Peek(R.EA);
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_pha()
/***************************************************************************

        pha

***************************************************************************/
{
  Poke(0x0100+R.S,R.A); R.S--;
}





void i_php()
/***************************************************************************

		  php

***************************************************************************/
{
  TEMP=(unsigned char)R.P.C
    + (((unsigned char)R.P.Z)<<1)
    + (((unsigned char)R.P.I)<<2)
    + (((unsigned char)R.P.D)<<3)
	 + (((unsigned char)R.P.B)<<4)
    + 0x20
    + (((unsigned char)R.P.V)<<6)
    + (((unsigned char)R.P.N)<<7);
  Poke(0x0100+R.S,(unsigned char)TEMP); R.S--;
}





void i_pla()
/***************************************************************************

        pla

***************************************************************************/
{
  R.S++; R.A=Peek(0x0100+R.S);
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_plp()
/***************************************************************************

		  plp

***************************************************************************/
{
  R.S++; TEMP=Peek(0x0100+R.S);
  if(TEMP & 0x01) R.P.C=1;
  else R.P.C=0;
  if(TEMP & 0x02) R.P.Z=1;
  else R.P.Z=0;
  if(TEMP & 0x04) R.P.I=1;
  else R.P.I=0;
  if(TEMP & 0x08) R.P.D=1;
  else R.P.D=0;
  if(TEMP & 0x10) R.P.B=1;
  else R.P.B=0;
  if(TEMP & 0x40) R.P.V=1;
  else R.P.V=0;
  if(TEMP & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_rol()
/***************************************************************************

		  rol

***************************************************************************/
{
  Mode[R.OP]();
  TEMP=Peek(R.EA);
  if(R.P.C==1) TEMP1=1;
  else TEMP1=0;
  if(TEMP & 0x80) R.P.C=1;
  else R.P.C=0;
  TEMP=TEMP<<1;
  TEMP += TEMP1;
  TEMP&=0xFF;
  Poke(R.EA,(unsigned char)TEMP);
  if(TEMP) R.P.Z=0;
  else R.P.Z=1;
  if(TEMP & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_rola()
/***************************************************************************

        rola

***************************************************************************/
{
  TEMP=R.A;
  if(R.P.C==1) TEMP1=1;
  else TEMP1=0;
  if(TEMP & 0x80) R.P.C=1;
  else R.P.C=0;
  TEMP=(TEMP<<1);
  TEMP += TEMP1;
  R.A=(unsigned char)TEMP;
  if(R.A==0) R.P.Z=1;
  else R.P.Z=0;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_ror()
/***************************************************************************

        ror

***************************************************************************/
{
  Mode[R.OP]();
  TEMP=Peek(R.EA);
  if(R.P.C==1) TEMP1=1;
  else TEMP1=0;
  if(TEMP & 0x01) R.P.C=1;
  else R.P.C=0;
  TEMP=TEMP>>1;
  if(TEMP1==1) TEMP+=0x80;
  TEMP&=0xFF;
  Poke(R.EA,(unsigned char)TEMP);
  if(TEMP) R.P.Z=0;
  else R.P.Z=1;
  if(TEMP & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_rora()
/***************************************************************************

        rora

***************************************************************************/
{
  TEMP=R.A;
  if(R.P.C==1) TEMP1=1;
  else TEMP1=0;
  if(TEMP & 0x01) R.P.C=1;
  else R.P.C=0;
  TEMP=TEMP>>1;
  if(TEMP1==1) TEMP+=0x80;
  R.A=(unsigned char)TEMP;
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_rti()
/***************************************************************************

		  rti

***************************************************************************/
{
  R.S++; TEMP=Peek(0x0100+R.S);
  if(TEMP & 0x01) R.P.C=1;
  else R.P.C=0;
  if(TEMP & 0x02) R.P.Z=1;
  else R.P.Z=0;
  if(TEMP & 0x04) R.P.I=1;
  else R.P.I=0;
  if(TEMP & 0x08) R.P.D=1;
  else R.P.D=0;
  if(TEMP & 0x10) R.P.B=1;
  else R.P.B=0;
  if(TEMP & 0x40) R.P.V=1;
  else R.P.V=0;
  if(TEMP & 0x80) R.P.N=1;
  else R.P.N=0;
  R.S++; R.PC.B.l=Peek(0x0100+R.S);
  R.S++; R.PC.B.h=Peek(0x0100+R.S);
}





void i_rts()
/***************************************************************************

        rts

***************************************************************************/
{
  R.S++; R.PC.B.l=Peek(0x0100+R.S);
  R.S++; R.PC.B.h=Peek(0x0100+R.S);
  R.PC.W++;
}





void i_sbc()
/***************************************************************************

        sbc

***************************************************************************/
{
  unsigned int temp;
  unsigned int t_data;
  unsigned char data;
  unsigned char z;
  Mode[R.OP]();
  data=Peek(R.EA);
  if(R.P.D==0)
  {
    t_data=(unsigned int)data+!R.P.C;
	 temp=(unsigned int)R.A-t_data;
	 z=(unsigned char)(temp&0xff);
	 if(z==0) R.P.Z=1;
	 else R.P.Z=0;
	 if(z>0x7F) R.P.N=1;
	 else R.P.N=0;
	 R.P.V=(~(R.A ^ t_data)) & (z ^ R.A) & 0x80;
    R.P.C=(temp>255)?0:1;
    R.A=z;
  }
  else
  {
	 int BCD1,BCD2;
	 t_data=Peek(R.EA);
	 BCD1=BCDtoDEC[R.A];
	 BCD2=BCDtoDEC[data];
    BCD1=BCD1-BCD2-!R.P.C;
    if(BCD1<0) BCD1=100-(-BCD1);
	 z=DECtoBCD[BCD1];
	 if(z==0) R.P.Z=1;
	 else R.P.Z=0;
	 R.P.C=(R.A < (t_data + !R.P.C))?0:1;
	 R.P.V=(z ^ R.A) & 0x80;
	 R.A=z;
  }
}





void i_sec()
/***************************************************************************

		  sec

***************************************************************************/
{
  R.P.C=1;
}





void i_sed()
/***************************************************************************

		  sed

***************************************************************************/
{
  R.P.D=1;
}





void i_sei()
/***************************************************************************

        sei

***************************************************************************/
{
  R.P.I=1;
}





void i_sta()
/***************************************************************************

        sta

***************************************************************************/
{
  Mode[R.OP]();
  Poke(R.EA,R.A);
}





void i_stx()
/***************************************************************************

		  stx

***************************************************************************/
{
  Mode[R.OP]();
  Poke(R.EA,R.X);
}





void i_sty()
/***************************************************************************

        sty

***************************************************************************/
{
  Mode[R.OP]();
  Poke(R.EA,R.Y);
}





void i_tax()
/***************************************************************************

        tax

***************************************************************************/
{
  Mode[R.OP]();
  R.X=R.A;
  if(R.X) R.P.Z=0;
  else R.P.Z=1;
  if(R.X & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_tay()
/***************************************************************************

		  tay

***************************************************************************/
{
  Mode[R.OP]();
  R.Y=R.A;
  if(R.Y) R.P.Z=0;
  else R.P.Z=1;
  if(R.Y & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_tsx()
/***************************************************************************

		  tsx

***************************************************************************/
{
  Mode[R.OP]();
  R.X=R.S;
  if(R.X) R.P.Z=0;
  else R.P.Z=1;
  if(R.X & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_txa()
/***************************************************************************

		  txa

***************************************************************************/
{
  Mode[R.OP]();
  R.A=R.X;
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_txs()
/***************************************************************************

		  txs

***************************************************************************/
{
  Mode[R.OP]();
  R.S=R.X;
  PokeFlag=TRUE;
}





void i_tya()
/***************************************************************************

		  tya

***************************************************************************/
{
  Mode[R.OP]();
  R.A=R.Y;
  if(R.A) R.P.Z=0;
  else R.P.Z=1;
  if(R.A & 0x80) R.P.N=1;
  else R.P.N=0;
}





void i_ERR()
/***************************************************************************

		  ERR - Undefined Instruction

***************************************************************************/
{
  Mode[R.OP]();
  if (CodeERR)                              // When Error handling active
  {
	PokeFlag=TRUE;
	SEnable=TRUE;
	if (hWndMonitor)
	{
		SEnable=TRUE;
		SMode=FALSE; 									// Enable Step Mode
	}
	else
	{
		Open_Monitor(ghWnd_Main,"Debugger ERR - Undefined Instruction"); 	// Open Debug Window
		display_registers();
		SEnable=FALSE;
	}
  }
}


void reset_cpu()
/***************************************************************************

		  reset_cpu

		  Sets up the virtual 6502 to it's power-on state

***************************************************************************/
{
  // Set registers to 0
  R.P.N=0; R.P.V=0; R.P.unused=1; R.P.B=0;
  R.P.D=0; R.P.I=0; R.P.Z=0; R.P.C=0;

  R.A=0; R.X=0; R.Y=0;

  R.S=0xff;

  R.PC.B.l=Peek(0xFFFC);
  R.PC.B.h=Peek(0xFFFD);
}


/***************************************************************************

		  irq_cpu

		  Run the CPU Interrupt request

***************************************************************************/
void irq_cpu()
{
  unsigned char LB;

  Poke(0x0100+R.S,(unsigned char)(R.PC.B.h)); R.S--;
  Poke(0x0100+R.S,(unsigned char)(R.PC.B.l)); R.S--;
  LB=(unsigned char)(R.P.C
	 + (((unsigned char)R.P.Z)<<1)
	 + (((unsigned char)R.P.I)<<2)
	 + (((unsigned char)R.P.D)<<3)
	 + (((unsigned char)R.P.B)<<4)
	 + 0x20
	 + (((unsigned char)R.P.V)<<6)
	 + (((unsigned char)R.P.N)<<7));
  Poke(0x0100+R.S,(unsigned char)LB); R.S--;
  R.P.I=1;
  R.PC.B.l=Peek(0xFFFE);
  R.PC.B.h=Peek(0xFFFF);
}


/***************************************************************************

		  nmi_cpu

		  Run the CPU NMI Interrupt request

***************************************************************************/
void nmi_cpu()
{
  unsigned char LB;

  Poke(0x0100+R.S,(unsigned char)(R.PC.B.h)); R.S--;
  Poke(0x0100+R.S,(unsigned char)(R.PC.B.l)); R.S--;
  LB=(unsigned char)(R.P.C
	 + (((unsigned char)R.P.Z)<<1)
	 + (((unsigned char)R.P.I)<<2)
	 + (((unsigned char)R.P.D)<<3)
	 + (((unsigned char)R.P.B)<<4)
	 + 0x20
	 + (((unsigned char)R.P.V)<<6)
	 + (((unsigned char)R.P.N)<<7));
  Poke(0x0100+R.S,(unsigned char)LB); R.S--;
  R.P.I=1;
  R.PC.B.l=Peek(0xFFFA);
  R.PC.B.h=Peek(0xFFFB);
}





void init_cpu()
/***************************************************************************

		  init_cpu

		  Set up the internal tables, etc. for the 6502 emulator.

***************************************************************************/
{
	int z;
	unsigned int index;
	for(z=0;z<0x100;z++)
	{
		BCDtoDEC[z]=(unsigned char)(((z>>4)&0xf)*10+(z&0xf));
		DECtoBCD[z]=(unsigned char)((((z%100)/10)<<4)|(z%10));
	}

	

	Ticks[0x00]=7; Inst[0x00]=i_brk; lstrcpy(Op[0x00],"BRK"); Mode[0x00]=addr_implied;
	Ticks[0x01]=6; Inst[0x01]=i_ora; lstrcpy(Op[0x01],"ORA"); Mode[0x01]=addr_indx;
	Ticks[0x02]=2; Inst[0x02]=i_ERR; lstrcpy(Op[0x02],"ERR"); Mode[0x02]=addr_immediate;
	Ticks[0x03]=1; Inst[0x03]=i_ERR; lstrcpy(Op[0x03],"ERR"); Mode[0x03]=addr_implied;
	Ticks[0x04]=2; Inst[0x04]=i_ERR; lstrcpy(Op[0x04],"ERR"); Mode[0x04]=addr_immediate;
	Ticks[0x05]=3; Inst[0x05]=i_ora; lstrcpy(Op[0x05],"ORA"); Mode[0x05]=addr_zpage;
	Ticks[0x06]=5; Inst[0x06]=i_asl; lstrcpy(Op[0x06],"ASL"); Mode[0x06]=addr_zpage;
	Ticks[0x07]=1; Inst[0x07]=i_ERR; lstrcpy(Op[0x07],"ERR"); Mode[0x07]=addr_implied;
	Ticks[0x08]=3; Inst[0x08]=i_php; lstrcpy(Op[0x08],"PHP"); Mode[0x08]=addr_implied;
	Ticks[0x09]=2; Inst[0x09]=i_ora; lstrcpy(Op[0x09],"ORA"); Mode[0x09]=addr_immediate;
	Ticks[0x0A]=2; Inst[0x0A]=i_asla; lstrcpy(Op[0x0A],"ASL A"); Mode[0x0A]=addr_implied;
	Ticks[0x0B]=1; Inst[0x0B]=i_ERR; lstrcpy(Op[0x0B],"ERR"); Mode[0x0B]=addr_implied;
	Ticks[0x0C]=4; Inst[0x0C]=i_ERR; lstrcpy(Op[0x0C],"ERR"); Mode[0x0C]=addr_abs;
	Ticks[0x0D]=4; Inst[0x0D]=i_ora; lstrcpy(Op[0x0D],"ORA"); Mode[0x0D]=addr_abs;
	Ticks[0x0E]=6; Inst[0x0E]=i_asl; lstrcpy(Op[0x0E],"ASL"); Mode[0x0E]=addr_abs;
	Ticks[0x0F]=1; Inst[0x0F]=i_ERR; lstrcpy(Op[0x0F],"ERR"); Mode[0x0F]=addr_implied;
	Ticks[0x10]=2; Inst[0x10]=i_bpl; lstrcpy(Op[0x10],"BPL"); Mode[0x10]=addr_relative;
	Ticks[0x11]=5; Inst[0x11]=i_ora; lstrcpy(Op[0x11],"ORA"); Mode[0x11]=addr_indy;
	Ticks[0x12]=2; Inst[0x12]=i_ERR; lstrcpy(Op[0x12],"ERR"); Mode[0x12]=addr_immediate;
	Ticks[0x13]=1; Inst[0x13]=i_ERR; lstrcpy(Op[0x13],"ERR"); Mode[0x13]=addr_implied;
	Ticks[0x14]=2; Inst[0x14]=i_ERR; lstrcpy(Op[0x14],"ERR"); Mode[0x14]=addr_immediate;
	Ticks[0x15]=4; Inst[0x15]=i_ora; lstrcpy(Op[0x15],"ORA"); Mode[0x15]=addr_zpagex;
	Ticks[0x16]=6; Inst[0x16]=i_asl; lstrcpy(Op[0x16],"ASL"); Mode[0x16]=addr_zpagex;
	Ticks[0x17]=1; Inst[0x17]=i_ERR; lstrcpy(Op[0x17],"ERR"); Mode[0x17]=addr_implied;
	Ticks[0x18]=2; Inst[0x18]=i_clc; lstrcpy(Op[0x18],"CLC"); Mode[0x18]=addr_implied;
	Ticks[0x19]=4; Inst[0x19]=i_ora; lstrcpy(Op[0x19],"ORA"); Mode[0x19]=addr_absy;
	Ticks[0x1A]=1; Inst[0x1A]=i_ERR; lstrcpy(Op[0x1A],"ERR"); Mode[0x1A]=addr_implied;
	Ticks[0x1B]=1; Inst[0x1B]=i_ERR; lstrcpy(Op[0x1B],"ERR"); Mode[0x1B]=addr_implied;
	Ticks[0x1C]=1; Inst[0x1C]=i_ERR; lstrcpy(Op[0x1C],"ERR"); Mode[0x1C]=addr_abs;
	Ticks[0x1D]=4; Inst[0x1D]=i_ora; lstrcpy(Op[0x1D],"ORA"); Mode[0x1D]=addr_absx;
	Ticks[0x1E]=7; Inst[0x1E]=i_asl; lstrcpy(Op[0x1E],"ASL"); Mode[0x1E]=addr_absx;
	Ticks[0x1F]=1; Inst[0x1F]=i_ERR; lstrcpy(Op[0x1F],"ERR"); Mode[0x1F]=addr_implied;
	Ticks[0x20]=6; Inst[0x20]=i_jsr; lstrcpy(Op[0x20],"JSR"); Mode[0x20]=addr_abs;
	Ticks[0x21]=6; Inst[0x21]=i_and; lstrcpy(Op[0x21],"AND"); Mode[0x21]=addr_indx;
	Ticks[0x22]=2; Inst[0x22]=i_ERR; lstrcpy(Op[0x22],"ERR"); Mode[0x22]=addr_immediate;
	Ticks[0x23]=1; Inst[0x23]=i_ERR; lstrcpy(Op[0x23],"ERR"); Mode[0x23]=addr_implied;
	Ticks[0x24]=3; Inst[0x24]=i_bit; lstrcpy(Op[0x24],"BIT"); Mode[0x24]=addr_zpage;
	Ticks[0x25]=3; Inst[0x25]=i_and; lstrcpy(Op[0x25],"AND"); Mode[0x25]=addr_zpage;
	Ticks[0x26]=5; Inst[0x26]=i_rol; lstrcpy(Op[0x26],"ROL"); Mode[0x26]=addr_zpage;
	Ticks[0x27]=1; Inst[0x27]=i_ERR; lstrcpy(Op[0x27],"ERR"); Mode[0x27]=addr_implied;
	Ticks[0x28]=4; Inst[0x28]=i_plp; lstrcpy(Op[0x28],"PLP"); Mode[0x28]=addr_implied;
	Ticks[0x29]=2; Inst[0x29]=i_and; lstrcpy(Op[0x29],"AND"); Mode[0x29]=addr_immediate;
	Ticks[0x2A]=2; Inst[0x2A]=i_rola; lstrcpy(Op[0x2A],"ROL A"); Mode[0x2A]=addr_implied;
	Ticks[0x2B]=1; Inst[0x2B]=i_ERR; lstrcpy(Op[0x2B],"ERR"); Mode[0x2B]=addr_implied;
	Ticks[0x2C]=4; Inst[0x2C]=i_bit; lstrcpy(Op[0x2C],"BIT"); Mode[0x2C]=addr_abs;
	Ticks[0x2D]=4; Inst[0x2D]=i_and; lstrcpy(Op[0x2D],"AND"); Mode[0x2D]=addr_abs;
	Ticks[0x2E]=6; Inst[0x2E]=i_rol; lstrcpy(Op[0x2E],"ROL"); Mode[0x2E]=addr_abs;
	Ticks[0x2F]=1; Inst[0x2F]=i_ERR; lstrcpy(Op[0x2F],"ERR"); Mode[0x2F]=addr_implied;
	Ticks[0x30]=2; Inst[0x30]=i_bmi; lstrcpy(Op[0x30],"BMI"); Mode[0x30]=addr_relative;
	Ticks[0x31]=5; Inst[0x31]=i_and; lstrcpy(Op[0x31],"AND"); Mode[0x31]=addr_indy;
	Ticks[0x32]=2; Inst[0x32]=i_ERR; lstrcpy(Op[0x32],"ERR"); Mode[0x32]=addr_immediate;
	Ticks[0x33]=1; Inst[0x33]=i_ERR; lstrcpy(Op[0x33],"ERR"); Mode[0x33]=addr_implied;
	Ticks[0x34]=2; Inst[0x34]=i_ERR; lstrcpy(Op[0x34],"ERR"); Mode[0x34]=addr_immediate;
	Ticks[0x35]=4; Inst[0x35]=i_and; lstrcpy(Op[0x35],"AND"); Mode[0x35]=addr_zpagex;
	Ticks[0x36]=6; Inst[0x36]=i_rol; lstrcpy(Op[0x36],"ROL"); Mode[0x36]=addr_zpagex;
	Ticks[0x37]=1; Inst[0x37]=i_ERR; lstrcpy(Op[0x37],"ERR"); Mode[0x37]=addr_implied;
	Ticks[0x38]=2; Inst[0x38]=i_sec; lstrcpy(Op[0x38],"SEC"); Mode[0x38]=addr_implied;
	Ticks[0x39]=4; Inst[0x39]=i_and; lstrcpy(Op[0x39],"AND"); Mode[0x39]=addr_absy;
	Ticks[0x3A]=1; Inst[0x3A]=i_ERR; lstrcpy(Op[0x3A],"ERR"); Mode[0x3A]=addr_implied;
	Ticks[0x3B]=1; Inst[0x3B]=i_ERR; lstrcpy(Op[0x3B],"ERR"); Mode[0x3B]=addr_implied;
	Ticks[0x3C]=4; Inst[0x3C]=i_ERR; lstrcpy(Op[0x3C],"ERR"); Mode[0x3C]=addr_abs;
	Ticks[0x3D]=4; Inst[0x3D]=i_and; lstrcpy(Op[0x3D],"AND"); Mode[0x3D]=addr_absx;
	Ticks[0x3E]=7; Inst[0x3E]=i_rol; lstrcpy(Op[0x3E],"ROL"); Mode[0x3E]=addr_absx;
	Ticks[0x3F]=1; Inst[0x3F]=i_ERR; lstrcpy(Op[0x3F],"ERR"); Mode[0x3F]=addr_implied;
	Ticks[0x40]=6; Inst[0x40]=i_rti; lstrcpy(Op[0x40],"RTI"); Mode[0x40]=addr_implied;
	Ticks[0x41]=6; Inst[0x41]=i_eor; lstrcpy(Op[0x41],"EOR"); Mode[0x41]=addr_indx;
	Ticks[0x42]=2; Inst[0x42]=i_ERR; lstrcpy(Op[0x42],"ERR"); Mode[0x42]=addr_immediate;
	Ticks[0x43]=1; Inst[0x43]=i_ERR; lstrcpy(Op[0x43],"ERR"); Mode[0x43]=addr_implied;
	Ticks[0x44]=2; Inst[0x44]=i_ERR; lstrcpy(Op[0x44],"ERR"); Mode[0x44]=addr_immediate;
	Ticks[0x45]=3; Inst[0x45]=i_eor; lstrcpy(Op[0x45],"EOR"); Mode[0x45]=addr_zpage;
	Ticks[0x46]=5; Inst[0x46]=i_lsr; lstrcpy(Op[0x46],"LSR"); Mode[0x46]=addr_zpage;
	Ticks[0x47]=1; Inst[0x47]=i_ERR; lstrcpy(Op[0x47],"ERR"); Mode[0x47]=addr_implied;
	Ticks[0x48]=3; Inst[0x48]=i_pha; lstrcpy(Op[0x48],"PHA"); Mode[0x48]=addr_implied;
	Ticks[0x49]=2; Inst[0x49]=i_eor; lstrcpy(Op[0x49],"EOR"); Mode[0x49]=addr_immediate;
	Ticks[0x4A]=2; Inst[0x4A]=i_lsra; lstrcpy(Op[0x4A],"LSR A"); Mode[0x4A]=addr_implied;
	Ticks[0x4B]=2; Inst[0x4B]=i_ERR; lstrcpy(Op[0x4B],"ERR"); Mode[0x4B]=addr_implied;
	Ticks[0x4C]=3; Inst[0x4C]=i_jmp; lstrcpy(Op[0x4C],"JMP"); Mode[0x4C]=addr_abs;
	Ticks[0x4D]=4; Inst[0x4D]=i_eor; lstrcpy(Op[0x4D],"EOR"); Mode[0x4D]=addr_abs;
	Ticks[0x4E]=6; Inst[0x4E]=i_lsr; lstrcpy(Op[0x4E],"LSR"); Mode[0x4E]=addr_abs;
	Ticks[0x4F]=1; Inst[0x4F]=i_ERR; lstrcpy(Op[0x4F],"ERR"); Mode[0x4F]=addr_implied;
	Ticks[0x50]=2; Inst[0x50]=i_bvc; lstrcpy(Op[0x50],"BVC"); Mode[0x50]=addr_relative;
	Ticks[0x51]=5; Inst[0x51]=i_eor; lstrcpy(Op[0x51],"EOR"); Mode[0x51]=addr_indy;
	Ticks[0x52]=2; Inst[0x52]=i_ERR; lstrcpy(Op[0x52],"ERR"); Mode[0x52]=addr_immediate;
	Ticks[0x53]=1; Inst[0x53]=i_ERR; lstrcpy(Op[0x53],"ERR"); Mode[0x53]=addr_implied;
	Ticks[0x54]=2; Inst[0x54]=i_ERR; lstrcpy(Op[0x54],"ERR"); Mode[0x54]=addr_immediate;
	Ticks[0x55]=4; Inst[0x55]=i_eor; lstrcpy(Op[0x55],"EOR"); Mode[0x55]=addr_zpagex;
	Ticks[0x56]=6; Inst[0x56]=i_lsr; lstrcpy(Op[0x56],"LSR"); Mode[0x56]=addr_zpagex;
	Ticks[0x57]=1; Inst[0x57]=i_ERR; lstrcpy(Op[0x57],"ERR"); Mode[0x57]=addr_implied;
	Ticks[0x58]=2; Inst[0x58]=i_cli; lstrcpy(Op[0x58],"CLI"); Mode[0x58]=addr_implied;
	Ticks[0x59]=4; Inst[0x59]=i_eor; lstrcpy(Op[0x59],"EOR"); Mode[0x59]=addr_absy;
	Ticks[0x5A]=1; Inst[0x5A]=i_ERR; lstrcpy(Op[0x5A],"ERR"); Mode[0x5A]=addr_implied;
	Ticks[0x5B]=1; Inst[0x5B]=i_ERR; lstrcpy(Op[0x5B],"ERR"); Mode[0x5B]=addr_implied;
	Ticks[0x5C]=4; Inst[0x5C]=i_ERR; lstrcpy(Op[0x5C],"ERR"); Mode[0x5C]=addr_abs;
	Ticks[0x5D]=4; Inst[0x5D]=i_eor; lstrcpy(Op[0x5D],"EOR"); Mode[0x5D]=addr_absx;
	Ticks[0x5E]=7; Inst[0x5E]=i_lsr; lstrcpy(Op[0x5E],"LSR"); Mode[0x5E]=addr_absx;
	Ticks[0x5F]=1; Inst[0x5F]=i_ERR; lstrcpy(Op[0x5F],"ERR"); Mode[0x5F]=addr_implied;
	Ticks[0x60]=6; Inst[0x60]=i_rts; lstrcpy(Op[0x60],"RTS"); Mode[0x60]=addr_implied;
	Ticks[0x61]=6; Inst[0x61]=i_adc; lstrcpy(Op[0x61],"ADC"); Mode[0x61]=addr_indx;
	Ticks[0x62]=2; Inst[0x62]=i_ERR; lstrcpy(Op[0x62],"ERR"); Mode[0x62]=addr_immediate;
	Ticks[0x63]=1; Inst[0x63]=i_ERR; lstrcpy(Op[0x63],"ERR"); Mode[0x63]=addr_implied;
	Ticks[0x64]=2; Inst[0x64]=i_ERR; lstrcpy(Op[0x64],"ERR"); Mode[0x64]=addr_immediate;
	Ticks[0x65]=3; Inst[0x65]=i_adc; lstrcpy(Op[0x65],"ADC"); Mode[0x65]=addr_zpage;
	Ticks[0x66]=5; Inst[0x66]=i_ror; lstrcpy(Op[0x66],"ROR"); Mode[0x66]=addr_zpage;
	Ticks[0x67]=1; Inst[0x67]=i_ERR; lstrcpy(Op[0x67],"ERR"); Mode[0x67]=addr_implied;
	Ticks[0x68]=4; Inst[0x68]=i_pla; lstrcpy(Op[0x68],"PLA"); Mode[0x68]=addr_implied;
	Ticks[0x69]=2; Inst[0x69]=i_adc; lstrcpy(Op[0x69],"ADC"); Mode[0x69]=addr_immediate;
	Ticks[0x6A]=2; Inst[0x6A]=i_rora; lstrcpy(Op[0x6A],"ROR A"); Mode[0x6A]=addr_implied;
	Ticks[0x6B]=1; Inst[0x6B]=i_ERR; lstrcpy(Op[0x6B],"ERR"); Mode[0x6B]=addr_implied;
	Ticks[0x6C]=5; Inst[0x6C]=i_jmp; lstrcpy(Op[0x6C],"JMP"); Mode[0x6C]=addr_indirect;
	Ticks[0x6D]=4; Inst[0x6D]=i_adc; lstrcpy(Op[0x6D],"ADC"); Mode[0x6D]=addr_abs;
	Ticks[0x6E]=6; Inst[0x6E]=i_ror; lstrcpy(Op[0x6E],"ROR"); Mode[0x6E]=addr_abs;
	Ticks[0x6F]=1; Inst[0x6F]=i_ERR; lstrcpy(Op[0x6F],"ERR"); Mode[0X6F]=addr_implied;
	Ticks[0x70]=2; Inst[0x70]=i_bvs; lstrcpy(Op[0x70],"BVS"); Mode[0x70]=addr_relative;
	Ticks[0x71]=5; Inst[0x71]=i_adc; lstrcpy(Op[0x71],"ADC"); Mode[0x71]=addr_indy;
	Ticks[0x72]=2; Inst[0x72]=i_ERR; lstrcpy(Op[0x72],"ERR"); Mode[0x72]=addr_immediate;
	Ticks[0x73]=1; Inst[0x73]=i_ERR; lstrcpy(Op[0x73],"ERR"); Mode[0x73]=addr_implied;
	Ticks[0x74]=2; Inst[0x74]=i_ERR; lstrcpy(Op[0x74],"ERR"); Mode[0x74]=addr_immediate;
	Ticks[0x75]=4; Inst[0x75]=i_adc; lstrcpy(Op[0x75],"ADC"); Mode[0x75]=addr_zpagex;
	Ticks[0x76]=2; Inst[0x76]=i_ror; lstrcpy(Op[0x76],"ROR"); Mode[0x76]=addr_zpagex;
	Ticks[0x77]=1; Inst[0x77]=i_ERR; lstrcpy(Op[0x77],"ERR"); Mode[0x77]=addr_implied;
	Ticks[0x78]=2; Inst[0x78]=i_sei; lstrcpy(Op[0x78],"SEI"); Mode[0x78]=addr_implied;
	Ticks[0x79]=4; Inst[0x79]=i_adc; lstrcpy(Op[0x79],"ADC"); Mode[0x79]=addr_absy;
	Ticks[0x7A]=1; Inst[0x7A]=i_ERR; lstrcpy(Op[0x7A],"ERR"); Mode[0x7A]=addr_implied;
	Ticks[0x7B]=1; Inst[0x7B]=i_ERR; lstrcpy(Op[0x7B],"ERR"); Mode[0x7B]=addr_implied;
	Ticks[0x7C]=4; Inst[0x7C]=i_ERR; lstrcpy(Op[0x7C],"ERR"); Mode[0x7C]=addr_abs;
	Ticks[0x7D]=4; Inst[0x7D]=i_adc; lstrcpy(Op[0x7D],"ADC"); Mode[0x7D]=addr_absx;
	Ticks[0x7E]=2; Inst[0x7E]=i_ror; lstrcpy(Op[0x7E],"ROR"); Mode[0x7E]=addr_absx;
	Ticks[0x7F]=1; Inst[0x7F]=i_ERR; lstrcpy(Op[0x7F],"ERR"); Mode[0x7F]=addr_implied;
	Ticks[0x80]=2; Inst[0x80]=i_ERR; lstrcpy(Op[0x80],"ERR"); Mode[0x80]=addr_immediate;
	Ticks[0x81]=6; Inst[0x81]=i_sta; lstrcpy(Op[0x81],"STA"); Mode[0x81]=addr_indx;
	Ticks[0x82]=2; Inst[0x82]=i_ERR; lstrcpy(Op[0x82],"ERR"); Mode[0x82]=addr_immediate;
	Ticks[0x83]=1; Inst[0x83]=i_ERR; lstrcpy(Op[0x83],"ERR"); Mode[0x83]=addr_implied;
	Ticks[0x84]=3; Inst[0x84]=i_sty; lstrcpy(Op[0x84],"STY"); Mode[0x84]=addr_zpage;
	Ticks[0x85]=3; Inst[0x85]=i_sta; lstrcpy(Op[0x85],"STA"); Mode[0x85]=addr_zpage;
	Ticks[0x86]=3; Inst[0x86]=i_stx; lstrcpy(Op[0x86],"STX"); Mode[0x86]=addr_zpage;
	Ticks[0x87]=1; Inst[0x87]=i_ERR; lstrcpy(Op[0x87],"ERR"); Mode[0x87]=addr_implied;
	Ticks[0x88]=2; Inst[0x88]=i_dey; lstrcpy(Op[0x88],"DEY"); Mode[0x88]=addr_implied;
	Ticks[0x89]=2; Inst[0x89]=i_ERR; lstrcpy(Op[0x89],"ERR"); Mode[0x89]=addr_immediate;
	Ticks[0x8A]=2; Inst[0x8A]=i_txa; lstrcpy(Op[0x8A],"TXA"); Mode[0x8A]=addr_implied;
	Ticks[0x8B]=1; Inst[0x8B]=i_ERR; lstrcpy(Op[0x8B],"ERR"); Mode[0x8B]=addr_implied;
	Ticks[0x8C]=4; Inst[0x8C]=i_sty; lstrcpy(Op[0x8C],"STY"); Mode[0x8C]=addr_abs;
	Ticks[0x8D]=4; Inst[0x8D]=i_sta; lstrcpy(Op[0x8D],"STA"); Mode[0x8D]=addr_abs;
	Ticks[0x8E]=4; Inst[0x8E]=i_stx; lstrcpy(Op[0x8E],"STX"); Mode[0x8E]=addr_abs;
	Ticks[0x8F]=1; Inst[0x8F]=i_ERR; lstrcpy(Op[0x8F],"ERR"); Mode[0x8F]=addr_implied;
	Ticks[0x90]=2; Inst[0x90]=i_bcc; lstrcpy(Op[0x90],"BCC"); Mode[0x90]=addr_relative;
	Ticks[0x91]=6; Inst[0x91]=i_sta; lstrcpy(Op[0x91],"STA"); Mode[0x91]=addr_indy;
	Ticks[0x92]=2; Inst[0x92]=i_ERR; lstrcpy(Op[0x92],"ERR"); Mode[0x92]=addr_immediate;
	Ticks[0x93]=1; Inst[0x93]=i_ERR; lstrcpy(Op[0x93],"ERR"); Mode[0x93]=addr_implied;
	Ticks[0x94]=4; Inst[0x94]=i_sty; lstrcpy(Op[0x94],"STY"); Mode[0x94]=addr_zpagex;
	Ticks[0x95]=4; Inst[0x95]=i_sta; lstrcpy(Op[0x95],"STA"); Mode[0x95]=addr_zpagex;
	Ticks[0x96]=4; Inst[0x96]=i_stx; lstrcpy(Op[0x96],"STX"); Mode[0x96]=addr_zpagey;
	Ticks[0x97]=1; Inst[0x97]=i_ERR; lstrcpy(Op[0x97],"ERR"); Mode[0x97]=addr_implied;
	Ticks[0x98]=2; Inst[0x98]=i_tya; lstrcpy(Op[0x98],"TYA"); Mode[0x98]=addr_implied;
	Ticks[0x99]=5; Inst[0x99]=i_sta; lstrcpy(Op[0x99],"STA"); Mode[0x99]=addr_absy;
	Ticks[0x9A]=2; Inst[0x9A]=i_txs; lstrcpy(Op[0x9A],"TXS"); Mode[0x9A]=addr_implied;
	Ticks[0x9B]=1; Inst[0x9B]=i_ERR; lstrcpy(Op[0x9B],"ERR"); Mode[0x9B]=addr_implied;
	Ticks[0x9C]=4; Inst[0x9C]=i_ERR; lstrcpy(Op[0x9C],"ERR"); Mode[0x9C]=addr_abs;
	Ticks[0x9D]=5; Inst[0x9D]=i_sta; lstrcpy(Op[0x9D],"STA"); Mode[0x9D]=addr_absx;
	Ticks[0x9E]=4; Inst[0x9E]=i_ERR; lstrcpy(Op[0x9E],"ERR"); Mode[0x9E]=addr_abs;
	Ticks[0x9F]=1; Inst[0x9F]=i_ERR; lstrcpy(Op[0x9F],"ERR"); Mode[0x9F]=addr_implied;
	Ticks[0xA0]=2; Inst[0xA0]=i_ldy; lstrcpy(Op[0xA0],"LDY"); Mode[0xA0]=addr_immediate;
	Ticks[0xA1]=6; Inst[0xA1]=i_lda; lstrcpy(Op[0xA1],"LDA"); Mode[0xA1]=addr_indx;
	Ticks[0xA2]=2; Inst[0xA2]=i_ldx; lstrcpy(Op[0xA2],"LDX"); Mode[0xA2]=addr_immediate;
	Ticks[0xA3]=1; Inst[0xA3]=i_ERR; lstrcpy(Op[0xA3],"ERR"); Mode[0xA3]=addr_implied;
	Ticks[0xA4]=3; Inst[0xA4]=i_ldy; lstrcpy(Op[0xA4],"LDY"); Mode[0xA4]=addr_zpage;
	Ticks[0xA5]=3; Inst[0xA5]=i_lda; lstrcpy(Op[0xA5],"LDA"); Mode[0xA5]=addr_zpage;
	Ticks[0xA6]=3; Inst[0xA6]=i_ldx; lstrcpy(Op[0xA6],"LDX"); Mode[0xA6]=addr_zpage;
	Ticks[0xA7]=1; Inst[0xA7]=i_ERR; lstrcpy(Op[0xA7],"ERR"); Mode[0xA7]=addr_implied;
	Ticks[0xA8]=2; Inst[0xA8]=i_tay; lstrcpy(Op[0xA8],"TAY"); Mode[0xA8]=addr_implied;
	Ticks[0xA9]=2; Inst[0xA9]=i_lda; lstrcpy(Op[0xA9],"LDA"); Mode[0xA9]=addr_immediate;
	Ticks[0xAA]=2; Inst[0xAA]=i_tax; lstrcpy(Op[0xAA],"TAX"); Mode[0xAA]=addr_implied;
	Ticks[0xAB]=1; Inst[0xAB]=i_ERR; lstrcpy(Op[0xAB],"ERR"); Mode[0xAB]=addr_implied;
	Ticks[0xAC]=4; Inst[0xAC]=i_ldy; lstrcpy(Op[0xAC],"LDY"); Mode[0xAC]=addr_abs;
	Ticks[0xAD]=4; Inst[0xAD]=i_lda; lstrcpy(Op[0xAD],"LDA"); Mode[0xAD]=addr_abs;
	Ticks[0xAE]=4; Inst[0xAE]=i_ldx; lstrcpy(Op[0xAE],"LDX"); Mode[0xAE]=addr_abs;
	Ticks[0xAF]=1; Inst[0xAF]=i_ERR; lstrcpy(Op[0xAF],"ERR"); Mode[0xAF]=addr_implied;
	Ticks[0xB0]=2; Inst[0xB0]=i_bcs; lstrcpy(Op[0xB0],"BCS"); Mode[0xB0]=addr_relative;
	Ticks[0xB1]=5; Inst[0xB1]=i_lda; lstrcpy(Op[0xB1],"LDA"); Mode[0xB1]=addr_indy;
	Ticks[0xB2]=2; Inst[0xB2]=i_ERR; lstrcpy(Op[0xB2],"ERR"); Mode[0xB2]=addr_immediate;
	Ticks[0xB3]=1; Inst[0xB3]=i_ERR; lstrcpy(Op[0xB3],"ERR"); Mode[0xB3]=addr_implied;
	Ticks[0xB4]=4; Inst[0xB4]=i_ldy; lstrcpy(Op[0xB4],"LDY"); Mode[0xB4]=addr_zpagex;
	Ticks[0xB5]=4; Inst[0xB5]=i_lda; lstrcpy(Op[0xB5],"LDA"); Mode[0xB5]=addr_zpagex;
	Ticks[0xB6]=4; Inst[0xB6]=i_ldx; lstrcpy(Op[0xB6],"LDX"); Mode[0xB6]=addr_zpagey;
	Ticks[0xB7]=1; Inst[0xB7]=i_ERR; lstrcpy(Op[0xB7],"ERR"); Mode[0xB7]=addr_implied;
	Ticks[0xB8]=2; Inst[0xB8]=i_clv; lstrcpy(Op[0xB8],"CLV"); Mode[0xB8]=addr_implied;
	Ticks[0xB9]=4; Inst[0xB9]=i_lda; lstrcpy(Op[0xB9],"LDA"); Mode[0xB9]=addr_absy;
	Ticks[0xBA]=2; Inst[0xBA]=i_tsx; lstrcpy(Op[0xBA],"TSX"); Mode[0xBA]=addr_implied;
	Ticks[0xBB]=1; Inst[0xBB]=i_ERR; lstrcpy(Op[0xBB],"ERR"); Mode[0xBB]=addr_implied;
	Ticks[0xBC]=4; Inst[0xBC]=i_ldy; lstrcpy(Op[0xBC],"LDY"); Mode[0xBC]=addr_absx;
	Ticks[0xBD]=4; Inst[0xBD]=i_lda; lstrcpy(Op[0xBD],"LDA"); Mode[0xBD]=addr_absx;
	Ticks[0xBE]=4; Inst[0xBE]=i_ldx; lstrcpy(Op[0xBE],"LDX"); Mode[0xBE]=addr_absy;
	Ticks[0xBF]=1; Inst[0xBF]=i_ERR; lstrcpy(Op[0xBF],"ERR"); Mode[0xBF]=addr_implied;
	Ticks[0xC0]=2; Inst[0xC0]=i_cpy; lstrcpy(Op[0xC0],"CPY"); Mode[0xC0]=addr_immediate;
	Ticks[0xC1]=6; Inst[0xC1]=i_cmp; lstrcpy(Op[0xC1],"CMP"); Mode[0xC1]=addr_indx;
	Ticks[0xC2]=2; Inst[0xC2]=i_ERR; lstrcpy(Op[0xC2],"ERR"); Mode[0xC2]=addr_immediate;
	Ticks[0xC3]=1; Inst[0xC3]=i_ERR; lstrcpy(Op[0xC3],"ERR"); Mode[0xC3]=addr_implied;
	Ticks[0xC4]=3; Inst[0xC4]=i_cpy; lstrcpy(Op[0xC4],"CPY"); Mode[0xC4]=addr_zpage;
	Ticks[0xC5]=3; Inst[0xC5]=i_cmp; lstrcpy(Op[0xC5],"CMP"); Mode[0xC5]=addr_zpage;
	Ticks[0xC6]=5; Inst[0xC6]=i_dec; lstrcpy(Op[0xC6],"DEC"); Mode[0xC6]=addr_zpage;
	Ticks[0xC7]=1; Inst[0xC7]=i_ERR; lstrcpy(Op[0xC7],"ERR"); Mode[0xC7]=addr_implied;
	Ticks[0xC8]=2; Inst[0xC8]=i_iny; lstrcpy(Op[0xC8],"INY"); Mode[0xC8]=addr_implied;
	Ticks[0xC9]=2; Inst[0xC9]=i_cmp; lstrcpy(Op[0xC9],"CMP"); Mode[0xC9]=addr_immediate;
	Ticks[0xCA]=2; Inst[0xCA]=i_dex; lstrcpy(Op[0xCA],"DEX"); Mode[0xCA]=addr_implied;
	Ticks[0xCB]=1; Inst[0xCB]=i_ERR; lstrcpy(Op[0xCB],"ERR"); Mode[0xCB]=addr_implied;
	Ticks[0xCC]=4; Inst[0xCC]=i_cpy; lstrcpy(Op[0xCC],"CPY"); Mode[0xCC]=addr_abs;
	Ticks[0xCD]=4; Inst[0xCD]=i_cmp; lstrcpy(Op[0xCD],"CMP"); Mode[0xCD]=addr_abs;
	Ticks[0xCE]=6; Inst[0xCE]=i_dec; lstrcpy(Op[0xCE],"DEC"); Mode[0xCE]=addr_abs;
	Ticks[0xCF]=1; Inst[0xCF]=i_ERR; lstrcpy(Op[0xCF],"ERR"); Mode[0xCF]=addr_implied;
	Ticks[0xD0]=2; Inst[0xD0]=i_bne; lstrcpy(Op[0xD0],"BNE"); Mode[0xD0]=addr_relative;
	Ticks[0xD1]=5; Inst[0xD1]=i_cmp; lstrcpy(Op[0xD1],"CMP"); Mode[0xD1]=addr_indy;
	Ticks[0xD2]=2; Inst[0xD2]=i_ERR; lstrcpy(Op[0xD2],"ERR"); Mode[0xD2]=addr_immediate;
	Ticks[0xD3]=1; Inst[0xD3]=i_ERR; lstrcpy(Op[0xD3],"ERR"); Mode[0xD3]=addr_implied;
	Ticks[0xD4]=2; Inst[0xD4]=i_ERR; lstrcpy(Op[0xD4],"ERR"); Mode[0xD4]=addr_immediate;
	Ticks[0xD5]=4; Inst[0xD5]=i_cmp; lstrcpy(Op[0xD5],"CMP"); Mode[0xD5]=addr_zpagex;
	Ticks[0xD6]=6; Inst[0xD6]=i_dec; lstrcpy(Op[0xD6],"DEC"); Mode[0xD6]=addr_zpagex;
	Ticks[0xD7]=1; Inst[0xD7]=i_ERR; lstrcpy(Op[0xD7],"ERR"); Mode[0xD7]=addr_implied;
	Ticks[0xD8]=2; Inst[0xD8]=i_cld; lstrcpy(Op[0xD8],"CLD"); Mode[0xD8]=addr_implied;
	Ticks[0xD9]=4; Inst[0xD9]=i_cmp; lstrcpy(Op[0xD9],"CMP"); Mode[0xD9]=addr_absy;
	Ticks[0xDA]=1; Inst[0xDA]=i_ERR; lstrcpy(Op[0xDA],"ERR"); Mode[0xDA]=addr_implied;
	Ticks[0xDB]=1; Inst[0xDB]=i_ERR; lstrcpy(Op[0xDB],"ERR"); Mode[0xDB]=addr_implied;
	Ticks[0xDC]=4; Inst[0xDC]=i_ERR; lstrcpy(Op[0xDC],"ERR"); Mode[0xDC]=addr_abs;
	Ticks[0xDD]=4; Inst[0xDD]=i_cmp; lstrcpy(Op[0xDD],"CMP"); Mode[0xDD]=addr_absx;
	Ticks[0xDE]=7; Inst[0xDE]=i_dec; lstrcpy(Op[0xDE],"DEC"); Mode[0xDE]=addr_absx;
	Ticks[0xDF]=1; Inst[0xDF]=i_ERR; lstrcpy(Op[0xDF],"ERR"); Mode[0xDF]=addr_implied;
	Ticks[0xE0]=2; Inst[0xE0]=i_cpx; lstrcpy(Op[0xE0],"CPX"); Mode[0xE0]=addr_immediate;
	Ticks[0xE1]=6; Inst[0xE1]=i_sbc; lstrcpy(Op[0xE1],"SBC"); Mode[0xE1]=addr_indx;
	Ticks[0xE2]=2; Inst[0xE2]=i_ERR; lstrcpy(Op[0xE2],"ERR"); Mode[0xE2]=addr_immediate;
	Ticks[0xE3]=1; Inst[0xE3]=i_ERR; lstrcpy(Op[0xE3],"ERR"); Mode[0xE3]=addr_implied;
	Ticks[0xE4]=3; Inst[0xE4]=i_cpx; lstrcpy(Op[0xE4],"CPX"); Mode[0xE4]=addr_zpage;
	Ticks[0xE5]=3; Inst[0xE5]=i_sbc; lstrcpy(Op[0xE5],"SBC"); Mode[0xE5]=addr_zpage;
	Ticks[0xE6]=5; Inst[0xE6]=i_inc; lstrcpy(Op[0xE6],"INC"); Mode[0xE6]=addr_zpage;
	Ticks[0xE7]=1; Inst[0xE7]=i_ERR; lstrcpy(Op[0xE7],"ERR"); Mode[0xE7]=addr_implied;
	Ticks[0xE8]=2; Inst[0xE8]=i_inx; lstrcpy(Op[0xE8],"INX"); Mode[0xE8]=addr_implied;
	Ticks[0xE9]=2; Inst[0xE9]=i_sbc; lstrcpy(Op[0xE9],"SBC"); Mode[0xE9]=addr_immediate;
	Ticks[0xEA]=2; Inst[0xEA]=i_nop; lstrcpy(Op[0xEA],"NOP"); Mode[0xEA]=addr_implied;
	Ticks[0xEB]=1; Inst[0xEB]=i_ERR; lstrcpy(Op[0xEB],"ERR"); Mode[0xEB]=addr_implied;
	Ticks[0xEC]=4; Inst[0xEC]=i_cpx; lstrcpy(Op[0xEC],"CPX"); Mode[0xEC]=addr_abs;
	Ticks[0xED]=4; Inst[0xED]=i_sbc; lstrcpy(Op[0xED],"SBC"); Mode[0xED]=addr_abs;
	Ticks[0xEE]=6; Inst[0xEE]=i_inc; lstrcpy(Op[0xEE],"INC"); Mode[0xEE]=addr_abs;
	Ticks[0xEF]=1; Inst[0xEF]=i_ERR; lstrcpy(Op[0xEF],"ERR"); Mode[0xEF]=addr_implied;
	Ticks[0xF0]=2; Inst[0xF0]=i_beq; lstrcpy(Op[0xF0],"BEQ"); Mode[0xF0]=addr_relative;
	Ticks[0xF1]=5; Inst[0xF1]=i_sbc; lstrcpy(Op[0xF1],"SBC"); Mode[0xF1]=addr_indy;
	Ticks[0xF2]=2; Inst[0xF2]=i_ERR; lstrcpy(Op[0xF2],"ERR"); Mode[0xF2]=addr_immediate;
	Ticks[0xF3]=1; Inst[0xF3]=i_ERR; lstrcpy(Op[0xF3],"ERR"); Mode[0xF3]=addr_implied;
	Ticks[0xF4]=2; Inst[0xF4]=i_ERR; lstrcpy(Op[0xF4],"ERR"); Mode[0xF4]=addr_immediate;
	Ticks[0xF5]=4; Inst[0xF5]=i_sbc; lstrcpy(Op[0xF5],"SBC"); Mode[0xF5]=addr_zpagex;
	Ticks[0xF6]=6; Inst[0xF6]=i_inc; lstrcpy(Op[0xF6],"INC"); Mode[0xF6]=addr_zpagex;
	Ticks[0xF7]=1; Inst[0xF7]=i_ERR; lstrcpy(Op[0xF7],"ERR"); Mode[0xF7]=addr_implied;
	Ticks[0xF8]=2; Inst[0xF8]=i_sed; lstrcpy(Op[0xF8],"SED"); Mode[0xF8]=addr_implied;
	Ticks[0xF9]=4; Inst[0xF9]=i_sbc; lstrcpy(Op[0xF9],"SBC"); Mode[0xF9]=addr_absy;
	Ticks[0xFA]=1; Inst[0xFA]=i_ERR; lstrcpy(Op[0xFA],"ERR"); Mode[0xFA]=addr_implied;
	Ticks[0xFB]=1; Inst[0xFB]=i_ERR; lstrcpy(Op[0xFB],"ERR"); Mode[0xFB]=addr_implied;
	Ticks[0xFC]=4; Inst[0xFC]=i_ERR; lstrcpy(Op[0xFC],"ERR"); Mode[0xFC]=addr_abs;
	Ticks[0xFD]=4; Inst[0xFD]=i_sbc; lstrcpy(Op[0xFD],"SBC"); Mode[0xFD]=addr_absx;
	Ticks[0xFE]=7; Inst[0xFE]=i_inc; lstrcpy(Op[0xFE],"INC"); Mode[0xFE]=addr_absx;
	Ticks[0xFF]=1; Inst[0xFF]=i_ERR; lstrcpy(Op[0xFF],"ERR"); Mode[0xFF]=addr_implied;


	/***************************************************************************

			  Pointer definition for PEEK/POKE functions
			  Pre-definition to small cpu and memory model
			  Sub address smaler 256byte is handled by function

	***************************************************************************/

	for(index=0;index<0x100;index++)
	{
		readMEMORY[index]= NULLREAD; // Default nothing to read
		writeMEMORY[index]= NULLWRITE; // Default nothing to write
	}
}

// Copy kernel page kPage = 0-7, rPage = RAM page (00-FF)
void CopyKPage( int kPage, int rPage )
{
	int index;
	int kBase = kPage<<8, rBase = rPage<<8; 

	if (kPage < 0 || kPage > 8 || rPage < 0 || rPage > 0xFF)
		return;

	for (index = 0; index < 256; index++)
		RAMptr[rBase++] = KernalRom[ kBase++];


}


void Init_Hardware()
{
	unsigned int index;

	
	//Map basic ROM
	if ( gBasicEnable )
	{
		readMEMORY[0xA0]=	ROMBASICREAD;
		readMEMORY[0xA1]=	ROMBASICREAD;
		readMEMORY[0xA2]=	ROMBASICREAD;
		readMEMORY[0xA3]=	ROMBASICREAD;
		readMEMORY[0xA4]=	ROMBASICREAD;
		readMEMORY[0xA5]=	ROMBASICREAD;
		readMEMORY[0xA6]=	ROMBASICREAD;
		readMEMORY[0xA7]=	ROMBASICREAD;
		readMEMORY[0xA8]=	ROMBASICREAD;
		readMEMORY[0xA9]=	ROMBASICREAD;
		readMEMORY[0xAA]=	ROMBASICREAD;
		readMEMORY[0xAB]=	ROMBASICREAD;
		readMEMORY[0xAC]=	ROMBASICREAD;
		readMEMORY[0xAD]=	ROMBASICREAD;
		readMEMORY[0xAE]=	ROMBASICREAD;
		readMEMORY[0xAF]=	ROMBASICREAD;
		readMEMORY[0xB0]=	ROMBASICREAD;
		readMEMORY[0xB1]=	ROMBASICREAD;
		readMEMORY[0xB2]=	ROMBASICREAD;
		readMEMORY[0xB3]=	ROMBASICREAD;
		readMEMORY[0xB4]=	ROMBASICREAD;
		readMEMORY[0xB5]=	ROMBASICREAD;
		readMEMORY[0xB6]=	ROMBASICREAD;
		readMEMORY[0xB7]=	ROMBASICREAD;
		readMEMORY[0xB8]=	ROMBASICREAD;
		readMEMORY[0xB9]=	ROMBASICREAD;
		readMEMORY[0xBA]=	ROMBASICREAD;
		readMEMORY[0xBB]=	ROMBASICREAD;
		readMEMORY[0xBC]=	ROMBASICREAD;
		readMEMORY[0xBD]=	ROMBASICREAD;
		readMEMORY[0xBE]=	ROMBASICREAD;
		readMEMORY[0xBF]=	ROMBASICREAD;
	}
	else
	{
		for(index=0xA0; index<=0xBF; index++)
		{
			readMEMORY[index]= RAMREAD; 
			writeMEMORY[index]= RAMWRITE;
		}

	}

	// SYN600 - OSI multiboard Monitor ROM
	// page 000 'H/D/M'   maps to $FF00 for a C2/C4 disk system
	// page 100 keypoller maps to $FD00 for a C2/C4 system
	// page 200 monitor   maps to $FE00 for a C2/C4 system
	// page 300 'C/W/M'   maps to $FF00 for a C2/C4 tape system
	// page 400 ??		  maps to $FC00 for a C1 system
	// page 500 keypoller maps to $FD00 (for a C1 system?)
	// page 600 monitor   maps to $FE00 (for a C1 system?)
	// page 700 'D/C/W/M' maps to $FF00 for a C1 system
	//
	// $FD00-FDFF = 500: SYN600 Page 1	 CEGMON: Page 5
	//              502:        Page 1
	//              505:        Page 1
	//
	// $FE00-FEFF = 500: SYN600 Page 2			 Page 6
	//              502:        Page 2
	//              505:        Page 2
	//
	// $FF00-FFFF = 500: SYN600 Page 3			 Page 7
	//              502:        Page 3
	//              505:        Page 0
	//
	//

	//Handle Kernel mapping
	if (gKernelType == SYN600) // 0 = SYN600, 1=CEGMON
	{
		
		readMEMORY[0xFD]=	ROMKERNALREAD;
		readMEMORY[0xFE]=	ROMKERNALREAD;
		readMEMORY[0xFF]=	ROMKERNALREAD;
		
		switch (gMachineType)
		{
			case C1P:
			case UK101:
			case C1PMF:
				CopyKPage(7, 0xFF);
				CopyKPage(6, 0xFE);
				CopyKPage(5, 0xFD);
				CopyKPage(4, 0xFC);
				readMEMORY[0xFC] = ROMKERNALREAD;
				break;

			case C2P:	//500 board
			case C4P:	//502 Board $FCxx is ACIA
				CopyKPage(3, 0xFF);
				CopyKPage(2, 0xFE);
				CopyKPage(1, 0xFD);
				break;

			case C4PMF:	//505 board
			case CUSTOM:
				CopyKPage(0, 0xFF);
				CopyKPage(2, 0xFE);
				CopyKPage(1, 0xFD);
				break;
		}
	} 
	else if ( gKernelType == CEGMON)
	{
		//start by copying kernel rom into F8xx ram.
		//may be overwritten later by I/O
		CopyKPage(0, 0xF8);
		CopyKPage(1, 0xF9);
		CopyKPage(2, 0xFA);
		CopyKPage(3, 0xFB);
		CopyKPage(4, 0xFC);
		CopyKPage(5, 0xFD);
		CopyKPage(6, 0xFE);
		CopyKPage(7, 0xFF);
		
		
		switch (gMachineType)
		{
			case C1P:
			case UK101:
			case C1PMF:
				readMEMORY[0xF8]=	ROMKERNALREAD;
				readMEMORY[0xF9]=	ROMKERNALREAD;
				readMEMORY[0xFA]=	ROMKERNALREAD;
				readMEMORY[0xFB]=	ROMKERNALREAD;
				readMEMORY[0xFC]=	ROMKERNALREAD;
				readMEMORY[0xFD]=	ROMKERNALREAD;
				readMEMORY[0xFE]=	ROMKERNALREAD;
				readMEMORY[0xFF]=	ROMKERNALREAD;


				break;
			case C2P:	//500 board
			case C4PMF:	//505 board

				//$F7xx is occupied by PIA
				//$FCxx is ACIA
				//NO INFO ON WHERE CEGMON MAPS ON THESE SYSTEMS
				//ASSUME $F600
				readMEMORY[0xF6]=	ROMKERNALREAD;
				readMEMORY[0xF8]=	ROMKERNALREAD;
				readMEMORY[0xF9]=	ROMKERNALREAD;
				readMEMORY[0xFA]=	ROMKERNALREAD;
				readMEMORY[0xFB]=	ROMKERNALREAD;
				//FCxx is ACIA
				readMEMORY[0xFD]=	ROMKERNALREAD;
				readMEMORY[0xFE]=	ROMKERNALREAD;
				readMEMORY[0xFF]=	ROMKERNALREAD;
				CopyKPage(4, 0xF6);
				break;
			case C4P:	//502 Board $FCxx is ACIA
				readMEMORY[0xF7]=	ROMKERNALREAD;
				readMEMORY[0xF8]=	ROMKERNALREAD;
				readMEMORY[0xF9]=	ROMKERNALREAD;
				readMEMORY[0xFA]=	ROMKERNALREAD;
				readMEMORY[0xFB]=	ROMKERNALREAD;
				//FCxx is ACIA
				readMEMORY[0xFD]=	ROMKERNALREAD;
				readMEMORY[0xFE]=	ROMKERNALREAD;
				readMEMORY[0xFF]=	ROMKERNALREAD;
				CopyKPage(4, 0xF7);
				break;
			case CUSTOM:
				break;

		}


	}
	
	
	//Enable disk controller
	if ( gDiskEnable || gMachineType == C4PMF || gMachineType == C1PMF )
	{
		readMEMORY[0xC0]=DISKREAD;			// Set DISK Hardware ports
		writeMEMORY[0xC0]=DISKWRITE;        // can be overwritten by ROM/RAM ext
		
	}


	if (RAMGDC == 1)
	{
		for(index=(GDCadr>>8);index<(GDCadr>>8)+32;index++)
		{
			readMEMORY[index]= RAMREAD;            // Defines 8k GDC memory
			writeMEMORY[index]= GDCWRITE;
		}
	}

	// Restore core Hardware ports , video memory

	switch (gMachineType)
	{

		case CUSTOM:   //??
		case C2P:	//500 board
		case C4P:	//502 Board
		case C4PMF:	//505 board
			
			//Color RAM
			readMEMORY[0xE0]=	VIDCREAD;  
			readMEMORY[0xE1]=	VIDCREAD;  
			readMEMORY[0xE2]=	VIDCREAD;  
			readMEMORY[0xE3]=	VIDCREAD;  
			readMEMORY[0xE4]=	VIDCREAD;  
			readMEMORY[0xE5]=	VIDCREAD;  
			readMEMORY[0xE6]=	VIDCREAD;  
			readMEMORY[0xE7]=	VIDCREAD;  

			writeMEMORY[0xE0]=	VIDCWRITE;
			writeMEMORY[0xE1]=	VIDCWRITE;
			writeMEMORY[0xE2]=	VIDCWRITE;
			writeMEMORY[0xE3]=	VIDCWRITE;
			writeMEMORY[0xE4]=	VIDCWRITE;
			writeMEMORY[0xE5]=	VIDCWRITE;
			writeMEMORY[0xE6]=	VIDCWRITE;
			writeMEMORY[0xE7]=	VIDCWRITE;

			//2K video
			readMEMORY[0xD4]=	VIDCREAD;  
			readMEMORY[0xD5]=	VIDCREAD;  
			readMEMORY[0xD6]=	VIDCREAD;  
			readMEMORY[0xD7]=	VIDCREAD;

			writeMEMORY[0xD4]=	VIDCWRITE;
			writeMEMORY[0xD5]=	VIDCWRITE;
			writeMEMORY[0xD6]=	VIDCWRITE;
			writeMEMORY[0xD7]=	VIDCWRITE;
			
			//Fall through  
		case C1P:
		case UK101:
		case C1PMF:
			//1K video
			readMEMORY[0xD0]=	VIDCREAD;  
			readMEMORY[0xD1]=	VIDCREAD;  
			readMEMORY[0xD2]=	VIDCREAD;  
			readMEMORY[0xD3]=	VIDCREAD;

			writeMEMORY[0xD0]=	VIDCWRITE;
			writeMEMORY[0xD1]=	VIDCWRITE;
			writeMEMORY[0xD2]=	VIDCWRITE;
			writeMEMORY[0xD3]=	VIDCWRITE;
	}

	//set keyboard, ACIA, I/O
	switch (gMachineType)
	{
		case C1P:    
		case UK101:  
		case C1PMF:  	

			readMEMORY[0xDF]=	KEYBREAD;
			readMEMORY[0xF0]=	ACIA1READ;
			writeMEMORY[0xDF]=	KEYBWRITE;
			writeMEMORY[0xF0]=	ACIA1WRITE;
			break;
		
		case CUSTOM:   //??
		case C2P:	//500 board
		case C4PMF:	//505 board
			readMEMORY[0xF7]=	PIAREAD;
			writeMEMORY[0xF7]=	PIAWRITE;

		case C4P:	//502 Board
			readMEMORY[0xFC]=	ACIA1READ;
			writeMEMORY[0xFC]=	ACIA1WRITE;
			readMEMORY[0xDF]=	KEYBREAD2;
			writeMEMORY[0xDF]=	KEYBWRITE2;
			readMEMORY[0xDE] = 	VIDCTRLRD;
			writeMEMORY[0xDE]=	VIDCTRLWD;
			break;
	} 



	// Set known extensions
	for(index=0;index<((RAMTOP+255)>>8);index++)
	{
		readMEMORY[index]= RAMREAD;            // Defines RAM locations
		writeMEMORY[index]= RAMWRITE;
	}

	if (ROMEX1size>0)
	{
		for(index=(ROMEX1>>8);index<((ROMEX1+ROMEX1size+255)>>8);index++)
		{
			readMEMORY[index]= RAMREAD;            // Defines ROM Extension memory
			writeMEMORY[index]= NULLWRITE;
		}
	}

	if (ROMEX2size>0)
	{
		for(index=(ROMEX2>>8);index<((ROMEX2+ROMEX2size+255)>>8);index++)
		{
			readMEMORY[index]= RAMREAD;            // Defines ROM Extension memory
			writeMEMORY[index]= NULLWRITE;
		}
	}
	if (ROMEX3size>0)
	{
		for(index=(ROMEX3>>8);index<((ROMEX3+ROMEX3size+255)>>8);index++)
		{
			readMEMORY[index]= RAMREAD;            // Defines ROM Extension memory
			writeMEMORY[index]= NULLWRITE;
		}
	}

	if ((RAMEX1size>0)&(RAMEX1<=(0xFFFF-RAMEX1size+1)))
	{
		for(index=(RAMEX1>>8);index<((RAMEX1+RAMEX1size)>>8);index++)
		{
			readMEMORY[index]= RAMREAD;            // Defines RAM Extension memory
			writeMEMORY[index]= RAMWRITE;
		}
	}

	if ((RAMEX2size>0)&(RAMEX2<=(0xFFFF-RAMEX2size+1)))
	{
		for(index=(RAMEX2>>8);index<((RAMEX2+RAMEX2size)>>8);index++)
		{
			readMEMORY[index]= RAMREAD;            // Defines RAM Extension memory
			writeMEMORY[index]= RAMWRITE;
		}
	}


}



