/***************************************************************************

		  6502.h

		  A 6502 emulator core
        (only standard 6502 code, ERR code length added)

        Functions that must be defined externally:

		  unsigned char Peek(unsigned int address);
             Returns the value stored at memory location 'address'.

		  void Poke(unsigned int address, unsigned char data);
             Stores value 'data' at location 'address'.


		  Variables that must be defined externally:

		  CPUMODEL R;
				 Used to store the CPU registers and current status.

***************************************************************************/

#ifndef _6502_H
#define _6502_H


/* Global Variables */

typedef struct /* Processor status register (flags) */
{
  unsigned C:1;          // Carry
  unsigned Z:1;          // Zero
  unsigned I:1;          // IRQ Disable
  unsigned D:1;          // Decimal
  unsigned B:1;          // Borrow
  unsigned unused:1;
  unsigned V:1;          // Overflow
  unsigned N:1;          // Negative
} P_REGISTER;

typedef union
{
  struct { unsigned char l,h; } B;
  unsigned int W;
} pair;


typedef struct
{
  unsigned char A;        /* Accumulator */
  unsigned char X;        /* X register */
  unsigned char Y;        /* Y register */
  unsigned char S;        /* Stack pointer (+256) */
  pair PC;       			  /* Program Counter */
  unsigned char OP;       /* Opcode */
  P_REGISTER P;  			  /* P register */
  unsigned int EA;        /* Effective address */
} CPUMODEL;

/* Definitions */
extern CPUMODEL R;

extern void (*readMEMORY[256])();      /* Array of pointers to read functions  */
extern void (*writeMEMORY[256])();     /* Array of pointers to write functions */

extern void (*Inst[256])();            /* Array of pointers to functions */
extern void (*Mode[256])();            /* Array of modes for each instruction */
extern int Ticks[256];                 /* Array of tick counts for each instr. */
extern char Op[256][6];

extern unsigned char readvalue;
extern unsigned char writevalue;
extern unsigned int memoryaddress;
extern int IRQ, NMI;					//Flag to set IRQ or NMI
extern unsigned int eff_adr;



extern class Acia g_SerialPort;
extern class DiskCtrl g_DiskPort;



int _6502(void); //main CPU call
void init_cpu(void);
void Init_Hardware();
void reset_cpu(void);
void irq_cpu();
void nmi_cpu();
unsigned char Peek(unsigned int address);
unsigned int Deek(unsigned int address);
void Poke(unsigned int address, unsigned char value);


void NULLREAD();
void RAMREAD();
void ROMBASICREAD();
void ROMKERNALREAD();
void ACIA1READ();
void ACIA2READ();
void VIDCREAD();
void NULLWRITE();
void RAMWRITE();
void ACIA1WRITE();
void ACIA2WRITE();
void VIDCWRITE();
void GDCWRITE();

unsigned int Disassemble(char *S, unsigned int A);



#endif //6502_H
