// 6821.h: interface for the PIA6821 class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PIA6821_H__EE7A53A0_A454_11D5_B5C9_00E01890870B__INCLUDED_)
#define AFX_PIA6821_H__EE7A53A0_A454_11D5_B5C9_00E01890870B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "6502.h"
#include <windows.h>
#include <TCHAR.H>


#define IS_CA2_HANDSHAKE()      ((ctrl_a & 0x30) == 0x20)
#define IS_CA2_PULSE_MODE()     ((ctrl_a & 0x38) == 0x28)
#define IS_CA2_TOGGLE_MODE()    ((ctrl_a & 0x38) == 0x20)

#define IS_CB2_HANDSHAKE()      ((ctrl_b & 0x30) == 0x20)
#define IS_CB2_PULSE_MODE()     ((ctrl_b & 0x38) == 0x28)
#define IS_CB2_TOGGLE_MODE()    ((ctrl_b & 0x38) == 0x20)

#define P_PORT_A        0
#define P_CTRL_A        1
#define P_PORT_B        2
#define P_CTRL_B        3

typedef struct tagPIA {
    BYTE  port_a;       /* output register, i.e. what has been written by
                           the CPU. input is assembled at read time */
    BYTE  ddr_a;        /* PIA Port A DDR */
    BYTE  ctrl_a;

    BYTE  port_b;
    BYTE  ddr_b;        /* PIA Port B DDR */
    BYTE  ctrl_b;

    int ca_state;
    int cb_state;
} piareg;


class PIA6821 : public tagPIA
{
public:
	PIA6821();
	virtual ~PIA6821();


protected:
	int m_nIRQ;
	BYTE m_bPortAValue;
	BYTE m_bPortBValue;

//implementation
public:

	//PIA6821 access based on memory address
	void Write( unsigned int address, unsigned char writevalue);
	void Read( unsigned int address, unsigned char *readvalue);
	void Reset();
	BOOL IsIRQSet() { return m_nIRQ ? TRUE : FALSE; };

//override 	
	virtual void WritePortA(BYTE value);
	virtual void WritePortB(BYTE value);
	virtual BYTE ReadPortA();
	virtual BYTE ReadPortB();
	virtual void SetCA1(BYTE value);
	virtual void SetCA2(BYTE value);
	virtual void SetCB1(BYTE value);
	virtual void SetCB2(BYTE value);

protected:
	void UpdateIRQ(); 

};

#endif // !defined(AFX_PIA6821_H__EE7A53A0_A454_11D5_B5C9_00E01890870B__INCLUDED_)
