
#include "keyboard.h"
#include "6502.h"
#include <stdio.h>
#include "fileio.h"
#include "w_osi.h"
#include "w_io.h"
#include "resource.h"
#include "monitor.h"	//PokeFlag

//globals
unsigned char far *keyscanptr = NULL; // ptr to RAMptr[0xdf00]      Write (Scan)
unsigned char far *keyptr = NULL;     // ptr to RAMptr[0xdf01]      Read (Key)
unsigned int gScan=0;				  // Current scan code pressed


//local data
unsigned int key_table[128][2];
unsigned int key_pressed[256];
BOOL bKeyboardInitialized = FALSE;



//OSI keyboard decode table reference
//  Columns read from $DF00, Rows selected by writing to $DF00
//	C2/C4 series has noninverted values (bit set when key pressed)
//  C1/Superboard has inverted values, bit cleared when key pressed.
//
//     7    6    5    4    3    2    1    0
// -----------------------------------------
// 7- 1/!  2/"  3/#  4/$  5/%  6/&  7/'
//
// 6- 8/(  9/)  0/@  :/*   -/=  rub
//
// 5- ./<   L    O    lf   cr
// 
// 4-  W    E    R    T	   Y    U    I
// 					 
// 3-  S    D    F    G	   H    J    K
// 
// 2-  X    C    V    B    N    M    ,/>
//
// 1-  Q    A    Z   spc   /   ;/+	 P
// 
// 0- rpt  ctl  esc            lsh  rsh  caps


/***************************************************************************
		  KEYBREAD         $DF00-DFFF
		  Returns byte value
***************************************************************************/
void KEYBREAD()
{
  switch (memoryaddress&0x01)
  {
	case 0x00:		 /// Keyboard read umlenken auf 0xDF01
		PCtoKeystate();
		readvalue=*keyptr;
		break;

	default:
		NULLREAD();
		break;
  }
}

void KEYBREAD2()
{
	if ((memoryaddress & 0x01) == 0)
	{
		KEYBREAD();
	}
	else
	{
		//C2/C4 sound latch read?
		NULLREAD();
	}
}

/***************************************************************************
		  KEYBOARDWRITE	$DF00-DFFF
		  Writes byte value
	Since OSI doesn't fully decode memory address, a write to any address
	in the page has the same effect. Also since all you are doing is writing
	to a latch, the memory can not be read.
***************************************************************************/

void KEYBWRITE()
{
	
	if (gMachineClass == C2Machine)
		RAMptr[ memoryaddress & 0xFF00] = (BYTE) (writevalue ^ 0xFF);
	else
		RAMptr[ memoryaddress & 0xFF00] = writevalue;
	
	PokeFlag = TRUE;
}

void KEYBWRITE2()
{
	if ( (memoryaddress & 0x0001) == 0)
	{
	
		KEYBWRITE();
	}
	else
	{
		//C2/C4  sound control
	}
}


void LoadKeyTable( LPCSTR szFile, LPSTR szError)
{
	// Load Keybord layout
	FILE *keyFile = NULL;
	int index;

	if ( !szFile || !*szFile || (keyFile = fopen(szFile,"rb"))==NULL ) 
		lstrcpy(szError,"Error opening Keyboard Layout in KEYFILE=");
	else
	{
		int c;
		BOOL bError = FALSE;

		for(index = 0; !bError && index < 128; index++)
		{
			c = fgetc(keyFile);
			if (c != EOF )
			{
				key_table[index][0]= c;
			}
			else
				bError = TRUE;
		}
		
		for( index = 0; !bError && index < 128; index++)
		{
			
			c = fgetc(keyFile);
			if (c != EOF)
			{
				key_table[index][1] = c;
			}
			else
				bError = TRUE;
		}

		fclose(keyFile);
		if (bError)
		{
			lstrcpy(szError,"Error reading Keyboard Layout in KEYFILE");
			strcat(szError, szFile );
		}

	}

	bKeyboardInitialized = TRUE;
}

/// IO feature Calls

void Clear_KeyState()           /// Clear Keyboard
{
	gScan=0;

	memset( key_pressed, 0xFF, (sizeof(key_pressed[0]) * 256) );
	
	if (keyptr)
		*keyptr=0xFF;

	//restore inverted caps lock setting 
	if ( (GetKeyState(VK_CAPITAL) & 0x01) == 0) 
		key_pressed[0xFE] &= 0xFE;
}

////////////////////////////////////////////////////////////////////////////
//
// Console Input - process special PC keys
void PCSpecialKeystate()           		

{

	// gScan is PC keyboard       		
	// <128 pressed, >128 released		
	if (gScan < 127)
	switch(gScan)
	{
		// Simulate Cursor and Copy feature of Ex_Eprom
		case 75: // Cursor left
			Clear_KeyState();
			key_pressed[0xFE]&=0xBF;
			gScan=51;
			PCtoKeystate();
			break;

		case 77: // Cursor right
			Clear_KeyState();
			key_pressed[0xFE]&=0xBF;
			gScan=52;
			PCtoKeystate();
			break;

		case 72: // Cursor up
			Clear_KeyState();
			key_pressed[0xFE]&=0xFD;
			gScan=27;
			PCtoKeystate();
			break;

		case 80: // Cursor down 
			Clear_KeyState();
			gScan=27;
			PCtoKeystate();
			break;

		case 82: // Insert Char 
			Clear_KeyState();
			key_pressed[0xFE]&=0xFD;
			gScan=28;
			PCtoKeystate();
			break;
		
		case 83: // Delete Char 
			Clear_KeyState();
			key_pressed[0xFE]&=0xFD;
			gScan=24;
			PCtoKeystate();
			break;
		
		default:
			break;
	}
	 										
	if(gScan>198) 
	{
		Clear_KeyState();
	}
}

//////////////////////////////////////////////////////////////////////////////////
// Routine called by memory read at $DF00 (DF00 shows write byte, DF01 shows read
//
// Map current keyboard scan code to OSI keyboard (if any)
// return current selected row(s) currently pressed keys
//
void PCtoKeystate()	// Console Poll
{
	int i, j, c;
     									
	BOOL bSet = (gScan & 0x80);

	gScan &= 0x7F;

	if ( gScan == 58) //special handling for caps -- invert actual state
	{
		//caps key - get actual state & invert
		 if ( GetKeyState(VK_CAPITAL) & 0x01) //low bit set = key toggled, light on
		 {
			//not pressed
			key_pressed[0xFE] |= 0x01;
		 }
		 else
		 {
			//pressed
			key_pressed[0xFE] &= 0xFE;
		 }
	}
	else if ( (gScan == 0x2A || gScan == 0x36) )
	{
		//shift key up doesn't always send messages for both keys
		if (IsNT())
		{
			//at least one key up occured, maybe both
			//Win95 doesn't support left vs right designators
			SHORT wState;
			wState = GetAsyncKeyState(VK_RSHIFT);
			TRACE("Right wState = %X", (int)wState);
			
			if ( (wState & 0x8000) == 0)
				key_pressed[0xFE] |= 0x02;
			else
				key_pressed[0xFE] &= ~0x02;

			wState = GetAsyncKeyState(VK_LSHIFT);
			TRACE("  Left wState = %X\n", (int)wState);

			if ( (wState & 0x8000) == 0)
				key_pressed[0xFE] |= 0x04;
			else
				key_pressed[0xFE] &= ~0x04;
		}
		else
		{
			if (bSet)
			{
			
				if ( (GetAsyncKeyState(VK_SHIFT) & 0x8000) == 0)
					key_pressed[0xFE] |= 0x06;
			}
			else
			{
				key_pressed[key_table[gScan][0]] &= key_table[gScan][1];
			}			
		}


	}

	else if( gScan && !bSet )	//key set
	{
			key_pressed[key_table[gScan][0]] &= key_table[gScan][1];  // set inverted pressed bit
	}
	else if (bSet)//key cleared
	{
			key_pressed[key_table[gScan][0]] |= ~key_table[gScan][1]; // clr inverted pressed bit
	}
	
	gScan = 0; //no need to process this scan code any longer
	

	//	scan the active rows on the keyboard matrix and return the columns with keys pressed
	c =0xFF;							// assume no columns
	i = 128;							// bits 0-7 indicate the rows
	while (i > 0) 
	{
		j = (~i | *keyscanptr) & 0xFF;	// isolate row
		if (j < 0xFF)					// row is active
			c &= key_pressed[j];		// accumulate column bits
		i = i >> 1;
	}

	//Invert result for C2/C4
	if (gMachineClass == C2Machine)
		*keyptr = (BYTE) (c ^ 0xFF);
	else
		*keyptr = (BYTE) c; 

}

//    key_table[0..128][0] is the index for the key_pressed[] array
//    The key_pressed[0..255] array takes its index from the $DF00 output port
//    key_table[0..128][1] is the 8 bit return mask from the osi keyboard
//
////////////////////////////////////////////////////////////////////////////
