#include "w_osi.h"
#include "resource.h"
#include "w_io.h"
#include "keyboard.h"
#include "fileio.h"
#include "disk_io.h"
#include "monitor.h"	//for buffer

#include <windowsx.h>

HBITMAP LoadCHARGENFile(HDC dcMem);	//forward declaration
HBITMAP SizeBitmap(HBITMAP hBMSrc, int nWidth, int nHeight, HDC hDC);


HBITMAP bitmap32 = NULL;			// font bitmap handle 32x32
HBITMAP bitmap64 = NULL;			// font bitmap handle 64x??
HBITMAP pixel_bitmap = NULL;		// pixel bitmap handle s,m,d

HDC font32dc = NULL; 				// font window hdc 32 wide
HDC font64dc = NULL;				// font window hdc 64 wide
HDC pixel_window = NULL;			// pixel window hdc small


int posx, posy;						// Screen print position

unsigned int VIDCadr=0xD000;		// default D000 VIDC Startadress l=0800
unsigned int GDCadr=0x8000;			// default 8000 GDC Startadress  l=2000
unsigned int RAMGDC = 0;			// Hires memory for GDC
int	gScreenMode = RESIZE_MASK;		// Start with 32x32x small
int gSoundCtrl = 0;					// Sound Control
int gColorMode = 0;					// which palette to use	0= b/w 1= color 1, 2=color2
int g64x16Mode = 0;					// UK101 64x16 mode
BOOL GEnable = FALSE;               // Enables 8000 Graphics display


HBITMAP hOld32Bitmap = NULL;
HBITMAP hOld64Bitmap = NULL;
HBITMAP hOldPixBitmap = NULL;

const TCHAR szScreenSection[]	= _T("Screen");
const TCHAR szRGBEntry[]		= _T("ColorPref");
const TCHAR szResizeEntry[]		= _T("FixedWidth");
const TCHAR sz64x16Mode[]       = _T("Screen64x16");
const TCHAR szCharSize[]		= _T("Char_size");

// OSI 540 color
// Each color memory location 4bits
// Colors are either inverted (black background, with color text)
// or normal (colored background with black text)
// bit 0 = normal/invert=1
// bit 1-3= color index

//inverted bright colors 
COLORREF Palette[16] =
{
	//(real sampled OSI colors)
	RGB(200,193,18),	//yellow
	RGB(255,120,178),   //red
	RGB(128,250,80),	//green
	RGB(245,200,60),	//olive green
	RGB(89,204,245),	//blue
	RGB(180,138,255),	//purple
	RGB(130,210,255),	//sky blue
	RGB(192,192,192),	//white
	// theoretical RGB colors
	RGB(255,255,0),		//yellow
	RGB(255,0,0),		//red
	RGB(0,255,0),		//green
	RGB(153,204,0),		//olive green
	RGB(0,0,255),		//blue
	RGB(192,0,255),		//purple
	RGB(128,192,255),	//sky blue
	RGB(255,255,255),	//white

};

//dark colors  (real sampled OSI colors)
COLORREF BkPalette[16] =
{
	//(real sampled OSI colors)
	RGB(40,50,0),	//yellow
	RGB(100,0,20),	//red
	RGB(0,92,0),	//green
	RGB(71,52,0),   //olive green
	RGB(0,46,92),	//blue
	RGB(49,0,174),	//purple
	RGB(0,56,130),	//sky blue
	RGB(4,3,4),		//black 

	// theoretical RGB colors
	RGB(0,0,0),
	RGB(0,0,0),
	RGB(0,0,0),
	RGB(0,0,0),
	RGB(0,0,0),
	RGB(0,0,0),
	RGB(0,0,0),
	RGB(0,0,0),
};

// OSI 540 Board
// D000-D7FF = display RAM
// D800-DDFF = NULLREAD
// DE00-DEFF = display latch write, NULLREAD or $5f? Read
// DF00-DFFF = KB
// E000-E7FF = 4bit color ram. low bits=ram, high bits = NULLREAD
void InitVideo()
{

	HBITMAP hSmallFont = NULL;
	int xSize = 1, ySize = 1;
	BOOL b64Mode = FALSE;

	//bitmap 0 = Foreground color 
	SetTextColor(main_window, RGB(0,0,0));
	//bitmap 1 = background color
	SetBkColor(main_window, RGB(255,255,255));


	if (pixel_window)
	{
		if (pixel_bitmap)
		{
			SelectObject(pixel_window, hOldPixBitmap);
			DeleteObject( pixel_bitmap);
			pixel_bitmap = NULL;
			hOldPixBitmap = NULL;
		}

		
		DeleteDC( pixel_window );
		pixel_window = NULL;
	}	
	
	if (font32dc)
	{
		if (bitmap32)
		{
			SelectObject(font32dc, hOld32Bitmap);
			DeleteObject( bitmap32);
			bitmap32 = NULL;
		}

		DeleteDC( font32dc );
		font32dc = NULL;
	}

	if (font64dc)
	{
		if (bitmap64)
		{
			SelectObject(font32dc, hOld64Bitmap);
			DeleteObject( bitmap64);
			bitmap32 = NULL;
		}

		DeleteDC( font64dc );
		font64dc = NULL;
	}


	font32dc = CreateCompatibleDC(main_window);
	switch(gCharSet)
	{
		case 0:	//small OSI font
			hSmallFont = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_CHARMAP8));
			break;
		case 1:	//small UK101 font
			hSmallFont = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_CHARMAP1));
			break;
		case 2:	//external file
			hSmallFont = LoadCHARGENFile(font32dc);
			if (!hSmallFont) //load failed, default to internal
				hSmallFont = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_CHARMAP8));	
			break;
	}

	if (!hSmallFont)
	{
		return;
	}

	
	//font bitmaps are always 128x128.

	if (gMachineClass == C2Machine || (gScreenMode & MODE64X16) )
	{
		//need font for 64 width
		b64Mode = TRUE;
		font64dc = CreateCompatibleDC(main_window);
	}
	
	
	switch (gScreenMode & SCREENSIZE_MASK)
	{
		case 0:
			xSize = 128; ySize =128; //8x8 cell size
			break;

		case 1:
			xSize = 256; ySize = 256; //16x16 cell size
			break;
		case 3:
		case 2:
			xSize = 384; ySize = 384; //24x24
			break;

	}

	if (b64Mode)
	{
		// Bit  3 & 2 control 32/64 mode.  00=32x32,  01=64x32,   10=64x16,   11=??
		switch ( (gScreenMode >>2) & SCREENSIZE_MASK)
		{
			case 0:
			case 1:
				bitmap64 = SizeBitmap(hSmallFont, xSize/2, ySize, font32dc);
				break;
			case 2:
			case 3:
				bitmap64 = SizeBitmap(hSmallFont, xSize/2, ySize, font32dc);
				break;
		}
		
		//alternate bitmap for quick C2 32/64 mode switches
		
		SelectObject(font64dc,bitmap64); 
	}

	if (xSize != 128 || ySize != 128)
	{
		bitmap32 = SizeBitmap(hSmallFont, xSize, ySize, font32dc);
		DeleteObject(hSmallFont);
		hSmallFont = NULL;

	}
	else
	{
		bitmap32 = hSmallFont;
		hSmallFont = NULL;	
	}


	
	SelectObject(font32dc, bitmap32);
	 
	// Generates Pixel compatible object for small res

	pixel_window = CreateCompatibleDC(main_window);
	pixel_bitmap = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_HIRES8));

	BOOL b2x = FALSE;

	//determine if screen is 2x wide
	if ( ((gScreenMode & MODE64X16) && !(gScreenMode & RESIZE_MASK)) ||
	      ((gScreenMode & WIDTH_MASK) && !(gScreenMode & RESIZE_MASK)) )
		b2x = TRUE;

	
	if ( ySize != 128 || b2x)
	{
		HBITMAP hNew = NULL;
		if (b2x)
			hNew = SizeBitmap(pixel_bitmap, ySize/8, ySize*2, pixel_window);
		else
			hNew = SizeBitmap(pixel_bitmap, ySize/16, ySize*2, pixel_window);
		if (hNew)
		{
			
			DeleteObject( pixel_bitmap);
			pixel_bitmap = hNew;
		}

	}

	hOldPixBitmap = (HBITMAP)  SelectObject(pixel_window, pixel_bitmap);


}
//
// OSI font arranged 
#define CHARGEN_SIZE 2048
HBITMAP LoadCHARGENFile(HDC dcMem)
{

	HBITMAP hBM = NULL;
	FILE* file;
	BYTE bmih[ sizeof( BITMAPINFOHEADER ) + 2 * sizeof(RGBQUAD)];
	BYTE Data[CHARGEN_SIZE];
	BYTE Bits[CHARGEN_SIZE];
	LPBITMAPINFO  pBMI = (LPBITMAPINFO) &bmih;

	static BYTE Mask[8] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
	static BYTE Invr[8] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };

	int r,x,y,z;

	if (!CHARGEN || !*CHARGEN)
		return NULL;

	// Load data
	if((file=fopen(CHARGEN,"rb"))==NULL)
	{ 
		lstrcpy(ERRORTEXT,"Error opening CHARGEN ROM ");
		lstrcat(ERRORTEXT, CHARGEN);
		return NULL;
	}
	else
	{
		int nSize = fread( Data, 1, CHARGEN_SIZE, file );
		fclose(file);
		if (nSize != CHARGEN_SIZE)
		{
			lstrcpy(ERRORTEXT,"Error reading CHARGEN ROM ");
			return NULL;
		}
	}

	//OSI characters stored sequentially, 1 byte/row, 8 rows per char.
	//Bitmap is stored 16 characters per row, 8 rows per character
	//bottom up order...
	x = 0; y = 7; r = 15;
	for( z = 0; z < 2048; z++)
	{
		BYTE o;

		o = Data[z];
		
		if (gRevFont)
		{
			o = 0;
			for (int l = 0; l < 8; l++)	//horizontally reverse image
			{
				o |=  (Data[z] & Mask[l]) ? Invr[l] : 0;
			}
		}
		
		Bits[r*128 + y*16 +x] = o;

		y--;
		if (y < 0)
		{
			y = 7;
			x += 1;
			if (x == 16)
			{
				x = 0;
				r--;	
			}
		
		}

	}

	ZeroMemory( &bmih, sizeof(bmih));
	
	pBMI->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	pBMI->bmiHeader.biWidth = 128;
	pBMI->bmiHeader.biHeight = 128;
	pBMI->bmiHeader.biPlanes = 1;
	pBMI->bmiHeader.biBitCount = 1;
	pBMI->bmiHeader.biCompression =	BI_RGB;
	pBMI->bmiHeader.biSizeImage = 0;
	pBMI->bmiHeader.biClrUsed = 2;
	pBMI->bmiHeader.biClrImportant = 2;

	pBMI->bmiColors[0].rgbRed = 0;
	pBMI->bmiColors[0].rgbGreen = 0;
	pBMI->bmiColors[0].rgbBlue = 0;
	pBMI->bmiColors[0].rgbReserved = 0;

	pBMI->bmiColors[1].rgbRed = 255;
	pBMI->bmiColors[1].rgbGreen = 255;
	pBMI->bmiColors[1].rgbBlue = 255;
	pBMI->bmiColors[1].rgbReserved = 0;


	hBM = CreateDIBitmap(dcMem, &pBMI->bmiHeader, CBM_INIT, Bits, pBMI, DIB_RGB_COLORS);  
	return hBM;


}

HBITMAP SizeBitmap(HBITMAP hBMSrc, int nWidth, int nHeight, HDC hDC)
{
	BITMAP bm;
	HDC hDCMem = NULL;
	HBITMAP hBMNew, hbmOld1, hbmOld2;
	int nOldMode;
	BOOL bOK;

	ZeroMemory( &bm, sizeof(BITMAP));

	if (!GetObject(hBMSrc,  sizeof(BITMAP), &bm))
		return NULL;

	//nWidth = bm.bmWidth * nTimesX;
	//nHeight = bm.bmHeight * nTimesY;

	hDCMem = CreateCompatibleDC(hDC);
	hBMNew = CreateCompatibleBitmap(hDCMem, nWidth, nHeight);
	if ( !hBMNew )
	{
		DeleteDC( hDCMem );
		return NULL;
	}
		
		
	hbmOld1 = (HBITMAP) SelectObject(hDCMem, hBMNew);
	hbmOld2 = (HBITMAP) SelectObject(hDC, hBMSrc);
	
	nOldMode = SetStretchBltMode(hDCMem, WHITEONBLACK);
	
	bOK = StretchBlt(hDCMem,0,0,nWidth,nHeight,hDC,0,0,bm.bmWidth,bm.bmHeight,SRCCOPY);

	SelectObject( hDC, hbmOld2);
	SelectObject( hDCMem, hbmOld1);
 	
	SetStretchBltMode(hDCMem, nOldMode);
	DeleteDC( hDCMem );

	/* //debug
	if (OpenClipboard(ghWnd_Main))
	{
		EmptyClipboard();
		SetClipboardData(CF_BITMAP, hBMNew);
		CloseClipboard(); 
	}
	*/

	return hBMNew;

}

void TerminateVideo()
{

	TCHAR szTemp[20];
	
	if (pixel_window)
	{
		if (pixel_bitmap)
		{
			SelectObject(pixel_window, hOldPixBitmap);
			DeleteObject( pixel_bitmap);
			pixel_bitmap = NULL;
			hOldPixBitmap = NULL;
		}

		
		DeleteDC( pixel_window );
		pixel_window = NULL;
	}
	
	
	if (font32dc)
	{
		if (bitmap32)
		{
			SelectObject(font32dc, hOld32Bitmap);
			DeleteObject( bitmap32);
			bitmap32 = NULL;
		}

		DeleteDC( font32dc );
		font32dc = NULL;
	}

	if (font64dc)
	{
		if (bitmap64)
		{
			SelectObject(font32dc, hOld64Bitmap);
			DeleteObject( bitmap64);
			bitmap32 = NULL;
		}

		DeleteDC( font64dc );
		font64dc = NULL;
	}

	//save screen size
	_itot( (gScreenMode &= SCREENSIZE_MASK), szTemp, 10);
	WritePrivateProfileString(szScreenSection,szCharSize, szTemp ,INIFILE);
	
}

void ReadVideoINI()
{

}


//font 8x8, 16x16, 24x24,
//
//font 8x16, 

// Screen Char Output
// X/Y-Offset = 2x char size


// Screen settings
// Bits 1 & 0 control target size, 00=256x256,01=512x512, 10=768x768, 11=??
// Bit  3 & 2 control 32/64 mode.  00=32x32,  01=64x32,   10=64x16,   11=??

// 0 = 32x32, 256
// 1 = 32x32  512
// 2 = 32x32  768
// 3 = undef  
// 4 = 64x32  256
// 5 = 64x32  512
// 6 = 64x32  768
// 7 = undef
// 8 = 64x16  256
// 9 = 64x16  512
// A = 64x16  768
// B = undef
// C = undef
// D = undef
// F = undef


void OSI_VIDC(unsigned int VIDA) 
								 
{
	int x,y,addr,xx,yy,tmp=0;
	int color;
	COLORREF clBack, clFront, clOldBack = 0, clOldFront = 0;
	BOOL bAspect = ((gScreenMode & RESIZE_MASK) != 0);
	BOOL bBigChar = (gScreenMode & WIDTH_MASK) == 0 ; //True when 32x32 mode
	
	addr = VIDA & 0x07FF;

	if (gMachineClass == C2Machine)
		gScreenMode |= 4;
	
	if (gMachineClass == C1Machine && (VIDA & 0x0FFF) > 0x03FF)
		return;
	else if ( (VIDA & 0x0FFF) > 0x07FF)
		return;

	VIDA = (VIDA & 0x0FFF) | 0xD000;

	if (gScreenMode & COLOR_MASK)
	{

		BOOL bInverted;
		color = RAMptr[ (VIDA & 0x0FFF) | 0xE000] & 0x0F; //color nibble

		bInverted = (color & 1);
		color = color >> 1;

		if (gColorMode == 2)
			color += 8;	//add 8 to get to "pure" RGB palette entries
		else if (gColorMode == 0)
			color = 15;	//force B&W color

		if (bInverted)
		{
			//black background, colored character
			clFront = BkPalette[color];//RGB(0,0,0);
			clBack = Palette[ color];

		}
		else
		{
			//colored background, black character
			clFront = Palette[ color ];
			clBack = BkPalette[color];//RGB(0,0,0);
		}

		//bitmap 0 = Foreground color 
		clOldFront = SetTextColor(main_window, clBack);
		//bitmap 1 = background color
		clOldBack = SetBkColor(main_window, clFront);
	}



	VIDCpoke=TRUE;

	switch (gScreenMode & (SCREENSIZE_MASK | MODE64X16 | OSI540) )
	{
		case 0x00: // 0 = 32x32, screen is 256 x 256 font is 256x256 cell is 8x8
			tmp=addr>>0x5;		// tmp = addr / 32          
			y=tmp<<0x3;         // y = tmp * 8
			y+=VBORDER;			// y = y+16

			x=addr%0x20;		// x = addr mod 32	
			x<<=0x3;			// x = x * 8
			x+=HBORDER;			// x = x + 12

			addr=RAMptr[VIDA];	// get char   	
			tmp=0x8;            // size = 8     
			xx=addr&0xF;        // fontx = addr & 15   ; font is 8x8 bitmap     
			xx<<=0x3;			// fontx = fontx * 8

			yy=addr>>0x4;       // fonty = addr / 16     
			yy<<=0x3;			// fonty = fonty * 8

			BitBlt(main_window, x, y, tmp, tmp, font32dc, xx, yy, MERGECOPY);
		break;

		case 0x01:	// 1 = 32x32  screen is 512x512 font is 256x256 cell is 16x16
			tmp=addr>>0x5;						
			y=tmp<<0x4;              
			y+=VBORDER;

			x=addr%0x20;             
			x<<=0x4;                 
			x+=HBORDER;

			addr=RAMptr[VIDA];	   	
			tmp=0x10;
			xx=addr & 0xF;
			xx<<=0x4;

			yy=addr>>0x4;
			yy<<=0x4;
			BitBlt(main_window,x,y,tmp,tmp,font32dc,xx,yy,MERGECOPY);
		break;

		case 0x02: // 2 = 32x32  screen is 768x768 font is 256x256 cell is 24x24
		case 0x03: //undef
			tmp=addr>>0x5;			//convert address to row & column for this screen size     
			y=tmp*24;        	//to offset in bitmap
			y+=VBORDER;

			x= addr % 0x20;    
			x *=24;
			x+=HBORDER;

			addr=RAMptr[VIDA];		//calculate offset into font	
			tmp=0x18;
			xx= (addr & 0xF) * 24;
			yy= (addr>>0x4) * 24;
			BitBlt(main_window,x,y,tmp,tmp,font32dc,xx,yy,MERGECOPY);

		break;

		case 0x04:  // 4 = 64x32  screen is 256x256  Font is 64x128 cell is 4x8
          		 // unless bAspect then screen is 512x256 cell is 8x8 when 64col
			if ( bBigChar || !bAspect)
			{
				//32x32 mode font is 128 x 128, cell is 8x8
				
				x=addr%0x40;		//x mod 64 = col
				
				if (x > ( bAspect ? 31 : 63) )
					break;			// > col 32 = nothing to draw
				
				x<<=0x3;			//x * 8 = horiz
				x+=HBORDER;			// add border
				
				
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp<<0x3;         //y * 8 = vert pos
				y+=VBORDER;			// add border

				addr=RAMptr[VIDA];	//get char 

				xx=addr&0xF;        //get font xpos 
				xx<<=0x3;			//*8 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x3;			// *8 = offset

				BitBlt(main_window, x, y, 8, 8, font32dc, xx, yy, MERGECOPY);

			}
			else
			{
				//64x32 mode
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp<<0x3;         //y * 8 = vert pos
				y+=VBORDER;			// add border

				x=addr%0x40;		//x mod 64 = col
				x<<=0x2;			//x * 4 = horiz
				x+=HBORDER;			// add border

				addr=RAMptr[VIDA];	//get char 
				tmp=0x10;  				
				xx=addr&0xF;        //get font xpos 
				xx<<=0x2;			//*4 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x3;			// *8 = offset

				BitBlt(main_window, x, y, 4, 8, font64dc, xx, yy, MERGECOPY);
			}
		break;

		case 0x05:	// 5 = 64x32 screen is 512x512
			if ( bBigChar || !bAspect)
			{
				//32 wide mode Font is 256x256 cell is 16x16
				x=addr%0x40;		//x mod 64 = col
				
				if (x > ((bAspect || bBigChar) ? 31 : 63))
					break;

				x<<=0x4;			//x * 16 = horiz
				x+=HBORDER;			// add border
				
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp<<0x4;         //y * 16 = vert pos
				y+=VBORDER;			// add border



				addr=RAMptr[VIDA];	//get char 
				xx=addr&0xF;        //get font xpos 
				xx<<=0x4;			//*16 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x4;			// *16 = offset

				BitBlt(main_window, x, y, 16, 16, font32dc, xx, yy, MERGECOPY);

			}
			else
			{
			    //   Font is 128x256 cell is 8x16
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp<<0x4;         //y * 16 = vert pos
				y+=VBORDER;			// add border

				x=addr%0x40;		//x mod 64 = col
				x<<=0x3;			//x * 8 = horiz
				x+=HBORDER;			// add border

				addr=RAMptr[VIDA];	//get char 
				xx=addr&0xF;        //get font xpos 
				xx<<=0x3;			//*8 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x4;			// *16 = offset

				BitBlt(main_window, x, y, 8, 16, font64dc, xx, yy, MERGECOPY);
			}

		break;
		

		case 0x06: // 6 = 64x32  screen is 768x768  
			if ( (gScreenMode & WIDTH_MASK) == 0 || !bAspect)
			{
				//Font is 384x384 cell is 24x24
				x=addr%0x40;		//x mod 64 = col
				if (x > ( bAspect ? 31 : 63))
					break;

				x *=24;			    //x * 24 = horiz
				x+=HBORDER;			// add border
				

				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp*24;           //y * 24 = vert pos
				y+=VBORDER;			// add border


				addr=RAMptr[VIDA];	//get char 
           
				xx=addr&0xF;        //get font xpos 
				xx *=24;			//*24 = offset

				yy=addr>>0x4;       //get font ypos        
				yy *= 24;			// *24 = offset

				BitBlt(main_window, x, y, 24, 24, font32dc, xx, yy, MERGECOPY);


			}
			else
			{
				//Font is 192x384 cell is 16x24

				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp*24;          //y * 24 = vert pos
				y+=VBORDER;			// add border

				x=addr%0x40;		//x mod 64 = col
				x*=12;			//x * 12 = horiz
				x+=HBORDER;			// add border

				addr=RAMptr[VIDA];	//get char 
          
				xx=addr&0xF;        //get font xpos 
				xx *=12;			//*12 = offset

				yy=addr>>0x4;       //get font ypos        
				yy *= 24;			// *24 = offset

				BitBlt(main_window, x, y, 12, 24, font64dc, xx, yy, MERGECOPY);
			}
		break;
		case 0x07: // 7 = undef

		case 0x08:  // 8 = 64x16  screen is 256x256  Font is 64x128 cell is 4x8

			if (bAspect)
			{
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp<<0x3;         //y * 8 = vert pos
				y+=VBORDER+0x48;	// add border 

				x=addr%0x40;		//x mod 64 = col
				x<<=0x2;			//x * 4 = horiz
				x+=0x0C;			// add border 

				addr=RAMptr[VIDA];	//get char 
			 					
				xx=addr&0xF;        //get font xpos 
				xx<<=0x2;			//*4 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x3;			// *8 = offset

				BitBlt(main_window, x, y, 4, 8, font64dc, xx, yy, MERGECOPY);

			}
			else
			{
				////64x16, screen is 512x256, font is 256x256 cell is 8x8 
				
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp<<0x3;         //y * 8 = vert pos
				y+=VBORDER;			// add border 

				x=addr%0x40;		//x mod 64 = col
				x<<=0x3;			//x * 8 = horiz
				x+=HBORDER;			// add border 

				addr=RAMptr[VIDA];	//get char 
			 					
				xx=addr&0xF;        //get font xpos 
				xx<<=0x3;			//*8 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x3;			// *8 = offset

				BitBlt(main_window, x, y, 8, 8, font32dc, xx, yy, MERGECOPY);
			}
		break;

		case 0x09:  // 9 = 64x16  screen is 512 x 512 Font is 128x256 cell is 8x16

			if (bAspect)
			{
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp<<0x4;         //y * 16 = vert pos
				y+=VBORDER+0x80;	// add border

				x=addr%0x40;		//x mod 64 = col
				x<<=0x3;			//x * 8 = horiz
				x+=HBORDER;			// add border  

				addr=RAMptr[VIDA];	//get char 
				xx=addr&0xF;        //get font xpos 
				xx<<=0x3;			//*8 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x4;			// *16 = offset

				BitBlt(main_window, x, y, 8, 16, font64dc, xx, yy, MERGECOPY);

			}
			else
			{
				//64x16, screen is 1024x256, font is 256x256 cell is 16x16 
				
				tmp=addr>>0x06;		//y = addr / 64
				y=tmp<<0x4;         //y * 16 = vert pos
				y+=VBORDER;			// add border

				x=addr%0x40;		//x mod 64 = col
				x<<=0x4;			//x * 16 = horiz
				x+=HBORDER;			// add border  

				addr=RAMptr[VIDA];	//get char 
				xx=addr&0xF;        //get font xpos 
				xx<<=0x4;			//*16 = offset

				yy=addr>>0x4;       //get font ypos        
				yy<<=0x4;			// *16 = offset

				BitBlt(main_window, x, y, 16, 16, font32dc, xx, yy, MERGECOPY);
			}
			break;

		case 0x0A:  // A = 64x16  screen is 768x768  Font is 192x384 cell is 16x24

			if (bAspect)
			{

				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp*24;			//y * 24 = vert pos
				y+=VBORDER+0xC0;	// add border 8 *24 + normal

				x=addr%0x40;		//x mod 64 = col
				x *=12;				//x * 12 = horiz
				x+=HBORDER;			// add border 

				addr=RAMptr[VIDA];	//get char 
            
				xx=addr&0xF;        //get font xpos 
				xx *=12;			//*12 = offset

				yy=addr>>0x4;       //get font ypos        
				yy *= 24;			// *24 = offset

				BitBlt(main_window, x, y, 12, 24, font64dc, xx, yy, MERGECOPY);
			}
			else
			{
				//64x16 screen is  1536x384 Font is 256 x 256, cell is 24x24
				tmp=addr>>0x06;		//y = addr / 64       
				y=tmp*24;           //y * 24 = vert pos
				y+=VBORDER;			// add border 8 *24 + normal

				x=addr%0x40;		//x mod 64 = col
				x *=24;				//x * 24 = horiz
				x+=HBORDER;			// add border 

				addr=RAMptr[VIDA];	//get char 
            
				xx=addr&0xF;        //get font xpos 
				xx *=24;			//*24 = offset

				yy=addr>>0x4;       //get font ypos        
				yy *= 24;			// *24 = offset

				BitBlt(main_window, x, y, 24, 24, font64dc, xx, yy, MERGECOPY);
			}
			break;


		case 0x0B:	// B = undef
		case 0x0C:	// C = undef
		case 0x0D:	// D = undef
		case 0x0E:	// E = undef
		case 0x0F:	// F = undef
			break;

	}

	if (gScreenMode & COLOR_MASK)
	{
		SetTextColor(main_window, clOldFront);
		SetBkColor(main_window, clOldBack);

	}

}


////////////////////////////////////////////////////////////////////////////

void OSI_GDC(unsigned int VIDA)            /// Screen Pixel Output

{
	int w,x,y,z;
	BOOL bAspect = ((gScreenMode & RESIZE_MASK) != 0);
	z = VIDA & 0x1FFF;             // 8 kB Graphic Memory
	
	w = (gScreenMode & SCREENSIZE_MASK);

	if ( gScreenMode & MODE64X16 ) //only can be set when C1 mode
		w |= 4;

	//determine if screen is 2x wide
	if ( ((gScreenMode & MODE64X16) && !bAspect) ||
	      ((gScreenMode & WIDTH_MASK) && !bAspect) )
		w |= 0x8;


	switch (w )
	{
		case 0x00:		// 32x32 display , Small screen (8x8)
			y=z>>0x5;
			y+=VBORDER;			
			x=z&0x1F;
			x<<=0x3;         		
			x+=HBORDER;
			BitBlt(main_window,x,y,8,1,pixel_window,0,RAMptr[VIDA],MERGECOPY);
			break;

		case 0x01:       // Med screen (16x16)
			y=z>>0x5;
			y=y<<1;
			y+=VBORDER;	
			x=z&0x1F;
			x<<=0x4;         			
			x+=HBORDER;
			BitBlt(main_window,x,y,16,2,pixel_window,0,RAMptr[VIDA]<<1,MERGECOPY);
			break;

		case 0x02:       //Large screen (24x24)             
			y=z>>0x5;
			y*=3;
			y+=VBORDER;						
			x=z&0x1F;
			x*=24;         			 
			x+=HBORDER;
			BitBlt(main_window,x,y,24,3,pixel_window,0,RAMptr[VIDA]*3,MERGECOPY);
			break;

		case 0x03:	//undef
			break;

			//C1 64x16
		case 0x04: //sm  screen 256x128 64x16
			y=z>>0x6;				//y=z/64
			y+=VBORDER+0x48;			
			x=z&0x1F;				//x=x mod 64
			x<<=0x3;         		
			x+=HBORDER;			 //(.5?)
			BitBlt(main_window,x,y,8,1,pixel_window,0,RAMptr[VIDA],MERGECOPY);
			break;		
		case 0x05: //med screen 512x256, 64x16 
			y=z>>0x5;
			y+=VBORDER+0x80;			
			x=z&0x1F;
			x<<=0x4;         		
			x+=HBORDER;
			BitBlt(main_window,x,y,16,1,pixel_window,0,RAMptr[VIDA]<<1,MERGECOPY);
			break;
		case 0x06: //large screen 768x384 64x16
			y=z/32;
			y*= (int)(float)1.5; //*1.5!
			y+=VBORDER+0xC0;			
			x=z&0x1F;
			x*=24;         		
			x+=HBORDER;
			BitBlt(main_window,x,y,24,2,pixel_window,0,RAMptr[VIDA]*3,MERGECOPY);
			break;
		case 0x07: //undef

			//2x wide
		case 0x08:	//screen 512x256 cell 16x8
			y=z>>0x5;
			y+=VBORDER;			
			x=z&0x1F;
			x<<=0x4;         		
			x+=HBORDER;
			BitBlt(main_window,x,y,16,1,pixel_window,0,RAMptr[VIDA],MERGECOPY);
			break;
		case 0x09:	//screen 1024x512 cell 32x16
			y=z>>0x5;
			y=y<<1;
			y+=VBORDER;	
			x=z&0x1F;
			x<<=0x5;         			
			x+=HBORDER;
			BitBlt(main_window,x,y,32,2,pixel_window,0,RAMptr[VIDA]<<1,MERGECOPY);
			break;				
		case 0x0A:	//screen 1536x768  cell 48x24
			y=z>>0x5;
			y=y<<2;
			y+=VBORDER;	
			x=z&0x1F;
			x*=48;         			
			x+=HBORDER;
			BitBlt(main_window,x,y,48,3,pixel_window,0,RAMptr[VIDA]*3,MERGECOPY);

		case 0x0B:
		
		case 0x0C:
		case 0x0D:
		case 0x0E:
		case 0x0F:	
			break;

	}
}

////////////////////////////////////////////////////////////////////////////

void restvid(void)           			/// Restore Video
{
	unsigned int index;
	for( index = VIDCadr; index < (VIDCadr+0x0400 + (gScreenMode & 4 ? 0x0400 : 0) ); index++) 
		OSI_VIDC(index);
	VIDCpoke=TRUE;
}

////////////////////////////////////////////////////////////////////////////

void restgdc(void)           			/// Restore Hires Video
{
	unsigned int index;
	if (GEnable)
	{
		VIDCpoke=FALSE;
		for( index = GDCadr; index < (GDCadr+0x2000); index++)
			if (RAMptr[index]!=0) 
				OSI_GDC(index);
	}
}

////////////////////////////////////////////////////////////////////////////

void clrvid(void)           			/// CLR Video

{
	unsigned int index;
	for( index = VIDCadr; index < (VIDCadr+0x0800); index++) 
		RAMptr[index]=32;
	
	if ( GEnable )
		for( index=GDCadr; index < (GDCadr+0x2000); index++) 
			RAMptr[index]=0;

}

////////////////////////////////////////////////////////////////////////////

void ToggleGraphik(void)           			/// Set Graphic Mode
{
	
	if (RAMGDC == 0)
		return;

	if (GEnable)
	{
		GEnable = FALSE;
		CheckMenuItem( GetMenu(ghWnd_Main), IDM_GRAPHIC, MF_UNCHECKED);

	}
	else
	{
		GEnable = TRUE;
		CheckMenuItem( GetMenu(ghWnd_Main), IDM_GRAPHIC, MF_CHECKED);
	}
	restvid();
}

////////////////////////////////////////////////////////////////////////////

void SetScreenIcons(HDC hdc)
{
	HDC hdcMemory;
	HBITMAP hbmp;
	BITMAP bm;

	hbmp = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_DISK1));
	GetObject(hbmp, sizeof(BITMAP), &bm);
	hdcMemory = CreateCompatibleDC(hdc);
	SelectObject(hdcMemory, hbmp);
	BitBlt(hdc,0x18,rcWinRect.bottom-27, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCCOPY);

	hbmp = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_DISK2));
	GetObject(hbmp, sizeof(BITMAP), &bm);
	SelectObject(hdcMemory, hbmp);
	BitBlt(hdc,0x58,rcWinRect.bottom-27, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCCOPY);

	hbmp = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_TAPE));
	GetObject(hbmp, sizeof(BITMAP), &bm);
	SelectObject(hdcMemory, hbmp);
	BitBlt(hdc,0x98,rcWinRect.bottom-27, bm.bmWidth, bm.bmHeight, hdcMemory, 0, 0, SRCCOPY);

	DeleteDC(hdcMemory);
	return;
}

////////////////////////////////////////////////////////////////////////////

void SetScreenBotton(HDC hdc)
{
	HDC hdcMemory_on, hdcMemory_off;
	HBITMAP hbmp_on, hbmp_off;
	BITMAP bm_on, bm_off;

	hbmp_on = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_ON));
	GetObject(hbmp_on, sizeof(BITMAP), &bm_on);
	hdcMemory_on = CreateCompatibleDC(hdc);
	SelectObject(hdcMemory_on, hbmp_on);
	
	hbmp_off = LoadBitmap(ghInstance, MAKEINTRESOURCE(IDB_OFF));
	GetObject(hbmp_off, sizeof(BITMAP), &bm_off);
	hdcMemory_off = CreateCompatibleDC(hdc);
	SelectObject(hdcMemory_off, hbmp_off);
	
	if(disk_1s)	
		BitBlt(hdc,0x41,rcWinRect.bottom-20, bm_on.bmWidth, bm_on.bmHeight, hdcMemory_on, 0, 0, SRCCOPY);
	else 
		BitBlt(hdc,0x41,rcWinRect.bottom-20, bm_off.bmWidth, bm_off.bmHeight, hdcMemory_off, 0, 0, SRCCOPY);

	if (disk_2s)	
		BitBlt(hdc,0x81,rcWinRect.bottom-20, bm_on.bmWidth, bm_on.bmHeight, hdcMemory_on, 0, 0, SRCCOPY);
	else 
		BitBlt(hdc,0x81,rcWinRect.bottom-20, bm_off.bmWidth, bm_off.bmHeight, hdcMemory_off, 0, 0, SRCCOPY);

	if(tape_1s) 
		BitBlt(hdc,0xC1,rcWinRect.bottom-20, bm_on.bmWidth, bm_on.bmHeight, hdcMemory_on, 0, 0, SRCCOPY);
	else 
		BitBlt(hdc,0xC1,rcWinRect.bottom-20, bm_off.bmWidth, bm_off.bmHeight, hdcMemory_off, 0, 0, SRCCOPY);

	DeleteDC(hdcMemory_on);
	DeleteDC(hdcMemory_off);
	return;
}

////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////


void Terminate(void)
{
	free(RAMptr);
	fflush(stdin);

	//TerminateDisk();

	TerminateVideo();

	// kill the application


	DeleteObject(debugfont);
	
}


//*******************************************************************
// VideoDlg - static member, handle Dialog messages
//
// paramaters:
//             hdlg          - The window handle for this message
//             msg	        - The message number
//             wparam        - The WPARAM parameter for this message
//             lparam        - The LPARAM parameter for this message
//
//*******************************************************************
BOOL CALLBACK  VideoDlg(HWND hDlg, UINT msg, WPARAM wParam, LPARAM )
{


	switch(msg)
	{
		case WM_INITDIALOG:
			{

				CenterWindow( hDlg, (HWND) ghWnd_Main);
				CheckDlgButton (hDlg, IDC_CB_MONOCHROME, (gColorMode == 0) ? 1 : 0);
				CheckDlgButton( hDlg, IDC_CB_64x16,  g64x16Mode ? 1 : 0);
				CheckDlgButton( hDlg, IDC_CB_RGB, (gColorMode == 2) ? 1 : 0);
				CheckDlgButton( hDlg, IDC_CB_ASPECT, (gScreenMode & RESIZE_MASK) ? 1 : 0);
				EnableWindow( GetDlgItem(hDlg, IDC_CB_RGB), IsDlgButtonChecked(hDlg,IDC_CB_MONOCHROME) ? 0 : 1);

				return FALSE; //haven't set focus to control
			}
			break;

		case WM_COMMAND:
			switch (GET_WM_COMMAND_ID(wParam, lParam))
			{
				case IDOK:
					{

						TCHAR szTemp[20];
						int nOldScreenMode = gScreenMode;
						int nOldColorMode = gColorMode;
						int nOld64x16Mode = g64x16Mode; 
						BOOL bRedraw = FALSE;


						g64x16Mode = IsDlgButtonChecked (hDlg, IDC_CB_64x16) ? 1 : 0;

						if ( IsDlgButtonChecked (hDlg, IDC_CB_MONOCHROME))
							gColorMode = 0;
						else
							gColorMode = IsDlgButtonChecked (hDlg, IDC_CB_RGB) ? 2 : 1;

						gScreenMode &= ~RESIZE_MASK;
						gScreenMode |= (IsDlgButtonChecked (hDlg, IDC_CB_ASPECT) ? RESIZE_MASK : 0);
						
						
						if (nOldColorMode != gColorMode)
						{
							//save color mode
							_itot( gColorMode, szTemp, 10);
							WritePrivateProfileString(szScreenSection,szRGBEntry, szTemp,INIFILE);
							bRedraw = TRUE;
						}
						
						if (g64x16Mode != nOld64x16Mode)
						{ 
							WritePrivateProfileString(szScreenSection,sz64x16Mode, 
								g64x16Mode ? _T("1") : _T("0") ,INIFILE); 
							bRedraw = TRUE;
							if (gMachineClass == C1Machine)
							{
								gScreenMode &= MODE64X16;
								if (g64x16Mode)
									gScreenMode |= MODE64X16; 
							}

						}
						
						if (nOldScreenMode != gScreenMode)
						{

							WritePrivateProfileString(szScreenSection,szResizeEntry, 
								(gScreenMode & RESIZE_MASK) ? _T("1"): _T("0") ,INIFILE);
							bRedraw = TRUE; 
						}
						
						if (bRedraw)
						{
							//notify display change
							PostMessage(ghWnd_Main, WM_SCREEN_UPD, 0, 0);
						}
						
						EndDialog(hDlg, IDOK);
						
					}
					break;
				case IDC_CB_MONOCHROME:
					if ( HIWORD(wParam) == BN_CLICKED)
					{
						EnableWindow( GetDlgItem(hDlg, IDC_CB_RGB), IsDlgButtonChecked(hDlg,IDC_CB_MONOCHROME) ? 0 : 1);
					}
					break;

				case IDCANCEL:
					EndDialog(hDlg, IDCANCEL);
					break;
				default:
					return FALSE;
			}
		default:
			return(FALSE);
	}
	return(FALSE);
}


