//
//
// Interface for disk control
/////////////////////////////////////////////////////////////////////////////

#ifndef _DISK_IO_H
#define _DISK_IO_H

#include "6821.h"


#define MAX_DRIVE 4
typedef struct tagDrive 
{
	BYTE*	pDiskBuf;	//buffer for disk image in memory
	BYTE*   pDataPtr;	//pointer in DiskBuf
	int     nTrack;		//current track
	int     nIndex;		//tracks index pulse
	int     nSector;	//track data current offset
	BOOL    bWProtect;  //Readonly = TRUE
	BOOL    bChanged;	//Disk Data Changed when TRUE
	TCHAR	szFile[_MAX_PATH]; //path to loaded file

} DISKDRIVE;

class DiskCtrl : public PIA6821
{
public:
	DiskCtrl(BOOL b8InchDrives = FALSE, int nDrives = 2);
	virtual ~DiskCtrl();


protected:
	void CreateBuffer();
	void FreeBuffer();
	static UINT CALLBACK FileHookProc( HWND hdlg, UINT uiMsg, 
							WPARAM wParam, LPARAM lParam );
	static void CALLBACK DiskTimerProc(HWND, UINT, UINT, DWORD);

//implementation
public:

	//DiskCtrl access based on memory address
	void Write( unsigned int address, unsigned char writevalue);
	void Read( unsigned int address, unsigned char *readvalue);
	void Reset();

	
	void OpenFile(int nDrive, TCHAR* szFile = NULL); //prompt user for disk file
	void CloseFile(int nDrive = 0);		//close file if open/changed
	void SaveFile(int nDrive, BOOL bPrompt =TRUE); //write changed disk image

	void SetINIFile(LPCTSTR szFile);			//Set .ini file
	BOOL SetDriveConfig(BOOL b8Inch = FALSE, int nNumDrives = 2);
	int  GetSelDrives() { return m_nSelDrive; };

	static DiskCtrl* GetDiskPtr() { return m_gDisk; };

//override base class 	
	virtual void WritePortA(BYTE value);
	virtual void WritePortB(BYTE value);
	virtual BYTE ReadPortA();
	virtual BYTE ReadPortB();

protected:

	DISKDRIVE m_Drive[MAX_DRIVE];	//disk drive data
 
	TCHAR	m_szIniFile[_MAX_PATH];
	
	int     m_nDriveSel;
	int     m_nSelDrive;
	int     m_nNumDrives;
	
	BOOL    m_b8Inch;

	BYTE    m_AciaCtrl;
	int     m_AciaData;

	UINT    m_nEvent;
	
	static DiskCtrl* m_gDisk;

};


extern int disk_1s, disk_2s;


#endif //_DISK_IO_H

